""" Data1d Loader class for GTC output

Current data1d loader can read the following output formats:
    ADIOS2 : da:ta1d.bp

Author: Wenhao Wang
Date: 10/27/2018

"""

import warnings

import numpy as np
# Currently, ADIOS2 on Titan requires MPI initialization. This will give
# trouble to non-MPI calls using this module
from mpi4py import MPI
# import adios2

from ..utility import parse_shape

class gtcData1d(object):
    """GTC Data1d Output Loader"""

    def __init__(self, fname, verbose=False):
        self.filename = fname
        self.load(verbose=verbose)

    def load(self, verbose):
        """ Generic loading method. """
        print("\n****** DATA1D PROCESSING *******\n")
        if ".out" in self.filename:
            warnings.warn("data1d.out is a deprecated output format."+
                          " Try to use ADIOS2 output file in new runs.")
            print("OLD OUTPUT file detected.")
            self.load_old(verbose=verbose)
        elif ".bp" in self.filename:
            print("ADIOS2 BP file detected.")
            self.load_bp(verbose=verbose)
        elif ".h5" in self.filename:
            print("HDF5 file detected.")
            self.load_h5(verbose=verbose)
        else:
            raise gtcData1dException('File format not supported!' +
                                       ' Make sure you keep the file'+
                                       ' extension unchanged.')

    def load_old(self,verbose,kind=0,fload=0,feload=0):
        """
        This reads data1d.out files into something that can be read by python more easily.
         Be careful as the time step in data1d.out is not the actual timestep in
         GTC simulations (data is written evert ndiag time steps).
         This scrip doesnt work for fast particles yet
    
        """

        ## default setting for input:
        kind=0
        fload=0
        feload=0


        datafile = open(self.filename, 'r')
        ndstep = int(datafile.readline())   # of time steps
        mpsi = int(datafile.readline())     # oof radial grids: mpsi+1
        nspecies = int(datafile.readline())  #  of species: ion, electron, EP, impuries
        nhybrid = int(datafile.readline())    # if hybrid model is loaded
        mpdata1d = int(datafile.readline())  # of variables per species
        nfield = int(datafile.readline()) #  of fields: phi, a_para, fluidne
        mfdata1d = int(datafile.readline())   #of variables per field: phi00, phi_rms, .. (not sure...)
        #print ndstep,mpsi,nspecies,nhybrid,mpdata1d,nfield,mfdata1d
    
        # define the data array
        data1di=np.zeros((ndstep,mpsi,mpdata1d),dtype=float)
        data1de=np.zeros((ndstep,mpsi,mpdata1d),dtype=float)
        data1df=np.zeros((ndstep,mpsi,mpdata1d),dtype=float)
        data1dfe=np.zeros((ndstep,mpsi,mpdata1d),dtype=float)
        field00=np.zeros((ndstep,mpsi,nfield),dtype=float)
        fieldrms=np.zeros((ndstep,mpsi,nfield),dtype=float)
    
        ## update values for parthist, modehist and fieldhist. 
        ## notice that this can be done after we define the data structure.
   
        #each time step (ndiag) GTC writes the physical quantities
        for t in range(0,ndstep):
    
            #read the first quantity (ions)
            for j in range(0,mpdata1d):
                for i in range(0,mpsi):
                    #data1di_fixt[i,j]=datafile.readline()
                    data1di[t,i,j]=datafile.readline()
    
            #check if there are more species
            ## TODO: a temporary setting to save species in data1de and data1df, regardless of their actual physical meaning.
            if nspecies==3:
                nhybrid=1
                fload=1
            elif nspecies==2:
                nhybrid=1
                fload=0

            if nspecies>1 :
                #check electron
                if nhybrid>0 :
                    for j in range(0,mpdata1d):
                        for i in range(0,mpsi):
                            #data1de_fixt[i,j]=datafile.readline()
                            data1de[t,i,j]=datafile.readline()
                #check for fast particles
                if fload>0 :
                    for j in range(0,mpdata1d):
                        for i in range(0,mpsi):
                            #data1df_fixt[i,j]=datafile.readline()
                            data1df[t,i,j]=datafile.readline()
                #check for fast electrons
                if feload>0 :
                    for j in range(0,mpdata1d):
                        for i in range(0,mpsi):
                            data1dfe[t,i,j]=datafile.readline()
    
            #read the 00 component
            for j in range(0,nfield):
                for i in range(0,mpsi):
                    field00[t,i,j]=datafile.readline()
            #read the RMS component
            for j in range(0,nfield):
                for i in range(0,mpsi):
                    fieldrms[t,i,j]=datafile.readline()
    
        datafile.close()

### old output for function
#        if kind==1:
#            return (data1di,ndstep,mpsi,nspecies,nhybrid,mpdata1d,nfield,mfdata1d)
#        elif kind==2:
#            return (data1de,ndstep,mpsi,nspecies,nhybrid,mpdata1d,nfield,mfdata1d)
#        elif kind==3:
#            return (data1df,ndstep,mpsi,nspecies,nhybrid,mpdata1d,nfield,mfdata1d)
#        elif kind==4:
#            return (data1dfe,ndstep,mpsi,nspecies,nhybrid,mpdata1d,nfield,mfdata1d)
#        elif kind==5:
#            return (field00,ndstep,mpsi,nspecies,nhybrid,mpdata1d,nfield,mfdata1d)
#        elif kind==6:
#            return (fieldrms,ndstep,mpsi,nspecies,nhybrid,mpdata1d,nfield,mfdata1d)


        ## create data structure similar to adios2 load_bp()

        self.__dict__['ndstep'] = ndstep   # of time steps
        self.__dict__['mpsi+1'] = mpsi   # of quantities per species: density,entropy,momentum,energy, fluxes
        self.__dict__['nspecies'] = nspecies # of species: ion, electron, EP, impuries
        self.__dict__['nhybrid'] = nhybrid   # of field variables: phi, a_par, fluidne
        self.__dict__['mpdata1d'] = mpdata1d    # of modes per field: (n,m)
        self.__dict__['nfield'] = nfield   # of quantities per field: rms, single spatial point, zonal components
        self.__dict__['mfdata1d'] = mfdata1d  # time step size

        self.__dict__['data1di'] = np.transpose(data1di,(0,2,1))  #partdata(ndstep,nspecies,mpdiag)
        self.__dict__['data1de'] = np.transpose(data1de,(0,2,1))  #partdata(ndstep,nspecies,mpdiag)
        self.__dict__['data1df'] = np.transpose(data1df,(0,2,1))  #partdata(ndstep,nspecies,mpdiag)
        self.__dict__['data1dfe'] = np.transpose(data1dfe,(0,2,1))  #partdata(ndstep,nspecies,mpdiag)
        self.__dict__['field00'] = np.transpose(field00,(0,2,1))  #fieldmode(ndstep,nfield,modes,2)
        self.__dict__['fieldrms'] = np.transpose(fieldrms,(0,2,1))  #fieldtime(ndstep,nfield,mfdiag)

        ## make a python dict to hold everything as well.
        self.variables = {}
 
        self.variables.update({'ndstep':self.ndstep})
        self.variables.update({'mpsi+1':self.__dict__['mpsi+1']})
        self.variables.update({'nspecies':self.nspecies})
        self.variables.update({'nhybrid':self.nhybrid})
        self.variables.update({'mpdata1d':self.mpdata1d})
        self.variables.update({'nfield':self.nfield})
        self.variables.update({'mfdata1d':self.mfdata1d})

        self.variables.update({'data1di':self.data1di})
        self.variables.update({'data1de':self.data1de})
        self.variables.update({'data1df':self.data1df})
        self.variables.update({'data1dfe':self.data1dfe})
        self.variables.update({'field00':self.field00})
        self.variables.update({'fieldrms':self.fieldrms})
 
        if verbose:
            print('The structure of this data is as follows ...')
            print(self.variables.keys())

        #raise NotImplemented('data1d.out loader is not implemented.')

    def load_bp(self, verbose):
        """Read Data1d data from the ADIOS data1d.bp file
           
           Generates the data properties with the same name as those in the
           data1d file.
        """
        import adios2

        fr = adios2.open(self.filename, "r", MPI.COMM_SELF)

        names = fr.available_variables()
        # print(names)

        assert 'fieldrms' in names.keys()
        time_count = int(names['fieldrms']['AvailableStepsCount'])

        for key in names.keys():
            if names[key]['SingleValue'] == 'true':
                self.__dict__[key] = fr.read(key)
            else:
                shape = parse_shape(names[key]['Shape'])
                start = np.zeros_like(shape)
                tcount_tmp = int(names[key]['AvailableStepsCount'])
                self.__dict__[key] = fr.read(key, start, shape, 0, tcount_tmp)

        #assert 'ndstep' in names.keys()
        self.ndstep = time_count
#        if names['ndstep'] != time_count:
        if self.ndstep != time_count:
            print("\nWaring:the simulation might be interrupted by certain reason, so that the actual time step number is not consistent to the prescribed step number. The variable 'ndstep' is reset to the actual step number.\n\n")
            self.__dict__['ndstep'] = time_count

        self.variables = names
        self.variables.update({'ndstep':self.ndstep})

        if verbose:
            print("Read variables: ", self.variables.keys())

    def load_h5(self, verbose):
        raise NotImplemented('data1d.h5 loader is not implemented.')
