""" Data3d Loader class for GTC output

Current data3d loader can read the following output formats:
    ADIOS2 : phi_3d.bp

Author: Lei Shi
Date: 06/20/2018

"""

import os
import warnings

import numpy as np
# Currently, ADIOS2 on Titan requires MPI initialization. This will give
# trouble to non-MPI calls using this module
from mpi4py import MPI
# import adios2

from ..utility import parse_shape

class gtcData3d(object):
    """GTC Data3d Output Loader"""

    def __init__(self, fname, verbose=False):
        self.filename = fname
        self.load(verbose=verbose)

    def load(self, verbose):
        """ Generic loading method. """
        print("\n****** DATA3D PROCESSING *******\n")
        if ".out" in self.filename:
#            warnings.warn("data3d.out is a deprecated output format."+
#                          " Try to use ADIOS2 output file in new runs.")
            print("data3d.out is a deprecated output format."+
                          " Try to use ADIOS2 output file in new runs.")

            print("OLD OUTPUT file detected.")
            ### load options:
            #   "all": load all data3d at once, similar to load_bp results
            #   "single": load one data3d with specified filename, 
            #             in visualizer the default _nstep=0, so it still 
            #              works with the similar variable shapes.
#            self.load_all_old("all",verbose=verbose)  
            self.load_all_old("single",verbose=verbose) 
        elif ".bp" in self.filename:
            print("ADIOS2 BP file detected.")
            self.load_bp(verbose=verbose)
        elif ".h5" in self.filename:
            print("HDF5 file detected.")
            self.load_h5(verbose=verbose)
        else:
            raise gtcData3dException('File format not supported!' +
                                       ' Make sure you keep the file'+
                                       ' extension unchanged.')

    def load_old(self, kind=0, verbose=False, fname='./data3d0000000.out'):
        #based in old script by Calvin Lau
        """
        This reads data3d.out files into something that can be read by python more easily.
        kind=0 reads all data
        kind=1 reads just profile
        kind=2 reads just pdf
        kind=3 reads just poloidal
        kind=4 reads just flux
    
    	This function was written by Calvin Lau
        """
        
        #kind = 0
    
        #try:  
        lines = open(fname, 'r').readlines()
        nspecies=int(lines[0])  # of species: ion, electron, EP, impuries
        nfield=int(lines[1])    # of field variables: phi, a_para, fluidne
        nvgrid=int(lines[2])    # of grids in energy and pitch
        mpsi=int(lines[3])      # of radial grids: mpsi+1
        mtgrid=int(lines[4])    # of poloidal grids
        mtoroidal=int(lines[5]) # of toroidal (parallel) grids
        tmax=float(lines[6])    #upper bound of temperature
        #print "nspecies=",nspecies,",","nfield=",nfield,",","nvgrid=",nvgrid,",","mpsi=",mpsi,",","mtgrid=",mtgrid,",","mtoroidal=",mtoroidal,",","tmax=",tmax
    
        sect1=mpsi*6*nspecies
        sect2=nvgrid*4*nspecies
        sect3=mtgrid*mpsi*(nfield+2)
        sect4=mtgrid*mtoroidal*nfield
    
        profileseries=np.zeros(sect1,dtype=float)
        pdfseries=np.zeros(sect2,dtype=float)
        poloidataseries=np.zeros(sect3,dtype=float)
        fluxdataseries=np.zeros(sect4,dtype=float)
        
        profile=np.zeros((mpsi,6,nspecies),dtype=float)
        pdf=np.zeros((nvgrid,4,nspecies),dtype=float)
        poloidata=np.zeros((mtgrid,mpsi,nfield+2),dtype=float)
        fluxdata=np.zeros((mtgrid,mtoroidal,nfield),dtype=float)
    
        ## this separates the files into the four sections as a series of float numbers
        if kind==0 or kind==1:
            for i in range(0,sect1):
                profileseries[i]=float(lines[7+i])
        if kind==0 or kind==2:
            for i in range(0,sect2): 
                pdfseries[i]=float(lines[7+sect1+i])
        if kind==0 or kind==3:
            for i in range(0,sect3):
                poloidataseries[i]=float(lines[7+sect1+sect2+i])
        if kind==0 or kind==4:
            for i in range(0,sect4): 
                fluxdataseries[i]=float(lines[7+sect1+sect2+sect3+i])
            
        ## this reorganizes the four sections into the data format we expect in idl
        if kind==0 or kind==1:
            for i in range(0,nspecies):
                for j in range(0,6):
                    for k in range(0,mpsi):
                        profile[k,j,i]=profileseries[k+(j*mpsi)+(i*mpsi*6)]
                        
        if kind==0 or kind==2:
            for i in range(0,nspecies):
                for j in range(0,4):
                    for k in range(0,nvgrid):
                        pdf[k,j,i]=pdfseries[k+(j*nvgrid)+(i*nvgrid*4)]
                        
        if kind==0 or kind==3:            
            for i in range(0,nfield+2):
                for j in range(0,mpsi):
                    for k in range(0,mtgrid):
                        poloidata[k,j,i]=poloidataseries[k+(j*mtgrid)+(i*mpsi*mtgrid)]
        
        if kind==0 or kind==4:    
            for i in range(0,nfield):
                for j in range(0,mtoroidal):
                    for k in range(0,mtgrid):
                        fluxdata[k,j,i]=fluxdataseries[k+(j*mtgrid)+(i*mtgrid*mtoroidal)]             

# old output codes:
        if kind==0:
            return (profile, pdf, poloidata,fluxdata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
        elif kind==1:
            return (profile,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
        elif kind==2:
            return (pdf,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
        elif kind==3:
            return (poloidata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)
        elif kind==4:
            return (fluxdata,nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax)

    
    def load_all_old(self, load_type, verbose):

        ## find all available snapshots in current directory.
        path, ftmp = os.path.split(self.filename)
        print(path)
        print(ftmp)

        self.step_id = []
        if load_type=="single":
            self.step_id.append(int(ftmp[4:11]))
            print("start loading single snapshot: ",ftmp)
        elif load_type=="all":
            for root, dirs, files in os.walk(path):
                for fname in files:
                    if fname.startswith("snap") and fname.endswith(".out"):
                        self.step_id.append(int(fname[4:11]))
                        #print(fname[4:11])
            self.step_id.sort()
            print("start loading all snapshots in the directory: ")
            print("snap_id: ",self.step_id)
        else:
            print("Error: the snapshot loader type is not correct.")
        #print(self.step_id)
        time_count = int(len(self.step_id))

        ## start to load all the snapshots available

        profile = []
        pdf = []
        poloidata = []
        fluxdata = []
        for fid in self.step_id:
            fname = path + '/snap' + str(fid).zfill(7) + '.out'
            profile_tmp, pdf_tmp, poloidata_tmp,fluxdata_tmp, \
                nspecies,nfield,nvgrid,mpsi,mtgrid,mtoroidal,tmax \
                = self.load_old(0,verbose,fname)
            #print(fid,'profile_tmp.shape =',np.shape(profile_tmp))
            profile.append(profile_tmp)
            #print(fid,'profile.shape =',np.shape(profile))
            pdf.append(pdf_tmp)
            poloidata.append(poloidata_tmp)
            fluxdata.append(fluxdata_tmp)
        #print(np.shape(profile))


        ## create data structure similar to adios2 load_bp()

        self.__dict__['nspecies'] = nspecies # of species: ion, electron, EP, impuries
        self.__dict__['nfield'] = nfield   # of field variables: phi, a_par, fluidne

        self.__dict__['nvgrid'] = nvgrid    # of grids in energy and pitch
        self.__dict__['mpsi'] = mpsi      # of radial grids: mpsi+1
        self.__dict__['mtgrid'] = mtgrid    # of poloidal grids
        self.__dict__['mtoroidal'] = mtoroidal # of toroidal (parallel) grids
        self.__dict__['tmax'] = tmax    #upper bound of temperature
        #print "nspecies=",nspecies,",","nfield=",nfield,",","nvgrid=",nvgrid,",","mpsi=",mpsi,",","mtgrid=",mtgrid,",","mtoroidal=",mtoroidal,",","tmax=",tmax
 
        self.__dict__['profile'] = np.transpose(profile,(0,3,2,1))  #profile(nspecies,6,mpsi)
        self.__dict__['pdf'] = np.transpose(pdf,(0,3,2,1))  #pdf(nspecies,4,nvgrid)
        self.__dict__['poloidata'] = np.transpose(poloidata,(0,3,2,1))  #poloidata(nfield+2,mpsi,mtgrid)
        self.__dict__['fluxdata'] = np.transpose(fluxdata,(0,3,2,1))  #fluxdata(nfield,mtoroidal,mtgrid)

        ## notice that we can still update the data even after we define the data structure, refer to history.py.

        #print('profile.shape =',np.shape(self.profile))
        
        ## make a python dict to hold everything as well.
        self.variables = {}

        self.variables.update({'step_id':self.step_id}) 
        self.variables.update({'nspecies':self.nspecies})
        self.variables.update({'nfield':self.nfield})
        self.variables.update({'nvgrid':self.nvgrid})
        self.variables.update({'mpsi':self.mpsi})
        self.variables.update({'mtoroidal':self.mtoroidal})
        self.variables.update({'tmax':self.tmax})
        self.variables.update({'profile':self.profile})
        self.variables.update({'pdf':self.pdf})
        self.variables.update({'poloidata':self.poloidata})
        self.variables.update({'fluxdata':self.fluxdata})

        if verbose:
            print('Read variables:')
            print(self.variables.keys())

#        raise NotImplemented('snapshot.out loader is not implemented.')


    def load_bp(self, verbose):
        """Read Snapshot data from the ADIOS phi_3d.bp file
           
           Generates the data properties with the same name as those in the
           equilibrium file.
        """
        import adios2

        fr = adios2.open(self.filename, "r", MPI.COMM_SELF)

        names = fr.available_variables()
        #print(names)
        assert 'step_id' in names.keys()

        time_count = int(names['step_id']['AvailableStepsCount'])
        #print('time_count=',time_count)

        for key in names.keys():
            if names[key]['SingleValue'] == 'true':
                self.__dict__[key] = fr.read(key)
            else:
                shape = parse_shape(names[key]['Shape'])
                start = np.zeros_like(shape)
                tcount_tmp = int(names[key]['AvailableStepsCount'])
                #print(key,"tcount_tmp=",tcount_tmp)
                self.__dict__[key] = fr.read(key, start, shape, 0, tcount_tmp)
                #print(key,"shape=",np.shape(self.__dict__[key]))

        #print('nstep.Shape=',names['step_id']['Shape'])
        #print('pdf.Shape=',names['pdf']['Shape'])
        #print('step_id_shape=',np.shape(self.step_id))
        #print('pdf_shape=',np.shape(self.pdf))

        self.variables = names

        ## TODO: we should output mtheta in bp files 
        self.mtheta_surf = np.shape(self.b_surf)[2]
        self.variables.update({'mtheta_surf':self.mtheta_surf})
         
        if verbose:
            print("Read variables: ", self.variables.keys())

    def load_h5(self, verbose):
        raise NotImplemented('equilibrium.h5 loader is not implemented.')
