""" Equilibrium Loader class for GTC output

Current equilibrium loader can read the following output formats:
    ADIOS2 : equilibrium.bp

Author: Lei Shi
Date: 06/05/2018

"""

import warnings

import numpy as np
# Currently, ADIOS2 on Titan requires MPI initialization. This will give
# trouble to non-MPI calls using this module
from mpi4py import MPI
# import adios2

from ..utility import parse_shape

class gtcEquilibrium(object):
    """GTC Equilibrium Output Loader"""

    def __init__(self, fname, verbose=False):
        self.filename = fname
        self.load(verbose=verbose)

    def load(self, verbose):
        """ Generic loading method. """
        print("\n****** EQUILIBRIUM PROCESSING *******\n")
        if ".out" in self.filename:
            warnings.warn("equilibrium.out is a deprecated output format."+
                          " Try to use ADIOS2 output file in new runs.")
            #print("OLD OUTPUT file detected.")
            self.load_old(verbose=verbose)
        elif ".bp" in self.filename:
            print("ADIOS2 BP file detected.")
            self.load_bp(verbose=verbose)
        elif ".h5" in self.filename:
            print("HDF5 file detected.")
            self.load_h5(verbose=verbose)
        else:
            raise gtcEquilibriumException('File format not supported!' +
                                          ' Make sure you keep the file'+
                                          ' extension unchanged.')

    def load_old(self, verbose):
        """
           Read Equilibrium data from the old equilibrium.out file

        """
        lines = open(self.filename, 'r').readlines()
        
        nrplot = int(lines[0])   # of time steps
        lsp = int(lines[1])   #
        countlines = 2

        pdata_lines = np.zeros(lsp*(nrplot+1))
        for i in range(lsp*(nrplot+1)):
            pdata_lines[i] = float(lines[countlines+i])
        countlines = countlines+lsp*(nrplot+1)
        
        pdata = pdata_lines.reshape((nrplot+1,-1))   #
        #print('pdata.shape=',np.shape(pdata))
        #print("Te=",pdata[4,:])
    # pdata(lsp,nrplot+1), current gtc: nrplot=29
    # 0: poloidal flux function
        datap = pdata[0,:]
    # 1: nomalized toroidal flux function
        torpsi_norm = pdata[1,:]
    # 2: minor radius
        Rminor = pdata[2,:]
    # 3: major radius
        Rmajor = pdata[3,:]
    # 4: Te
        Te0 = pdata[4,:]
    # 5: -d(ln(Te))/dr
        dlgTe_dpsi_neg = pdata[5,:]
    # 6: ne
        ne0 = pdata[6,:]
    # 7: -d(ln(ne))/dr
        dlgne_dpsi_neg = pdata[7,:]
    # 8: Ti
        Ti0 = pdata[8,:]
    # 9: -d(ln(Ti))/dr
        dlgTi_dpsi_neg = pdata[9,:]
    # 10: ni
        ni0 = pdata[10,:]
    # 11: -d(ln(ni))/dr
        dlgni_dpsi_neg = pdata[11,:]
    # 12: Tf
        Tf0 = pdata[12,:]
    # 13: -d(ln(Tf))/dr
        dlgtf_dpsi_neg = pdata[13,:]
    # 14: nf
        nf0 = pdata[14,:]
    # 15: -d(ln(nf))/dr
        dlgnf_dpsi_neg = pdata[15,:]
    # 16: zeff
        Zeff = pdata[16,:]
    # 17: toroidal roation
        u_tor = pdata[17,:]
    # 18: radial electric field
        Er = pdata[18,:]
    # 19: q
        q = pdata[19,:]
    # 20: d(ln(q))/dr
        dlgq_dpsi = pdata[20,:]
    # 21: g
        g = pdata[21,:]
    # 22: p
        P = pdata[22,:]
    # 23: r: rpsi
        r_1d = pdata[23,:]
    # 24: toroidal flux: torpsi
        torpsi = pdata[24,:]
    # 25: radial grid: rgpsi
        rg = pdata[25,:]
    # 26: inversion of torpsi: psitor
        psi_tor = pdata[26,:]
    # 27: inversion of rgpsi: psirg
        psi_rg = pdata[27,:]
    # 28: error of spline cos
        cos_err = pdata[28,:]
    # 29: error of spline sin
        sin_err = pdata[29,:]

    # TODO: output major R and minor R from GTC
        Rminor = r_1d/max(r_1d)
        Rmajor = r_1d
 

        nplot = int(lines[countlines])
        npsi = int(lines[countlines+1])
        ntheta = int(lines[countlines+2])   #
        countlines = countlines+3

    ### convert spdata into each 2d variables
        spdata_lines = np.zeros(npsi*ntheta*(nplot+2))
        for i in range(npsi*ntheta*(nplot+2)):
            spdata_lines[i] = float(lines[countlines+i])
        countlines = countlines+npsi*ntheta*(nplot+2)

        spdata = spdata_lines.reshape((nplot+2,ntheta,-1))
        #print('spdata.shape=',np.shape(spdata))
    #  spdata(npsi,ntheta,nplot+2), current gtc: nplot=5
    # 0: x
        R = spdata[0,:,:]
    # 1: z
        Z = spdata[1,:,:]
    # 2: b
        B = spdata[2,:,:]
    # 3: J
        Jacobian = spdata[3,:,:]
    # 4: i
        I = spdata[4,:,:]
    # 5: zeta2phi
        zeta2phi = spdata[5,:,:]
    # 6: del
        delta = spdata[6,:,:]

        ## create data structure similar to adios2 load_bp()

        # pdata variable set 
        self.__dict__['nrplot'] = nrplot+1   # 
        self.__dict__['lsp'] = lsp   # 

        self.__dict__['psi'] = datap
        self.__dict__['torpsi_norm'] = torpsi_norm
        self.__dict__['Rminor'] = Rminor
        self.__dict__['Rmajor'] = Rmajor
        self.__dict__['Te0'] = Te0
        self.__dict__['dlgTe_dpsi_neg'] = dlgTe_dpsi_neg
        self.__dict__['ne0'] = ne0
        self.__dict__['dlgne_dpsi_neg'] = dlgne_dpsi_neg
        self.__dict__['Ti0'] = Ti0
        self.__dict__['dlgTi_dpsi_neg'] = dlgTi_dpsi_neg
        self.__dict__['ni0'] = ni0
        self.__dict__['dlgni_dpsi_neg'] = dlgni_dpsi_neg
        self.__dict__['Tf0'] = Tf0
        self.__dict__['dlgtf_dpsi_neg'] = dlgtf_dpsi_neg
        self.__dict__['nf0'] = nf0
        self.__dict__['dlgnf_dpsi_neg'] = dlgnf_dpsi_neg
        self.__dict__['Zeff'] = Zeff
        self.__dict__['u_tor'] = u_tor
        self.__dict__['Er'] = Er
        self.__dict__['q'] = q
        self.__dict__['dlgq_dpsi'] = dlgq_dpsi
        self.__dict__['g'] = g
        self.__dict__['P'] = P
        self.__dict__['r_1d'] = r_1d
        self.__dict__['torpsi'] = torpsi
        self.__dict__['rg'] = rg
        self.__dict__['psi_tor'] = psi_tor
        self.__dict__['psi_rg'] = psi_rg
        self.__dict__['cos_err'] = cos_err
        self.__dict__['sin_err'] = sin_err

        # spdata variable set
        self.__dict__['nplot'] = nplot+2   # 
        self.__dict__['npsi'] = npsi   # 
        self.__dict__['ntheta'] = ntheta   # 

        self.__dict__['R'] = R   # 
        self.__dict__['Z'] = Z   # 
        self.__dict__['B'] = B   # 
        self.__dict__['Jacobian'] = Jacobian   # 
        self.__dict__['I'] = I   # 
        self.__dict__['nu'] = zeta2phi   # 
        self.__dict__['delta'] = delta   # 
        
        ## update values for parthist, modehist and fieldhist. 
        ## notice that this can be done after we define the data structure.

        ## make a python dict to hold everything as well.
        self.variables = {}

        # pdata set
        self.variables.update({'nrplot':self.nrplot})
        self.variables.update({'lsp':self.lsp})

        self.variables.update({'psi':self.psi})
        self.variables.update({'torpsi_norm':self.torpsi_norm})
        self.variables.update({'Rminor':self.Rminor})
        self.variables.update({'Rmajor':self.Rmajor})
        self.variables.update({'Te0':self.Te0})
        self.variables.update({'dlgTe_dpsi_neg':self.dlgTe_dpsi_neg})
        self.variables.update({'ne0':self.ne0})
        self.variables.update({'dlgne_dpsi_neg':self.dlgne_dpsi_neg})
        self.variables.update({'Ti0':self.Ti0})
        self.variables.update({'dlgTi_dpsi_neg':self.dlgTi_dpsi_neg})
        self.variables.update({'ni0':self.ni0})
        self.variables.update({'dlgni_dpsi_neg':self.dlgni_dpsi_neg})
        self.variables.update({'Tf0':self.Tf0})
        self.variables.update({'dlgtf_dpsi_neg':self.dlgtf_dpsi_neg})
        self.variables.update({'nf0':self.nf0})
        self.variables.update({'dlgnf_dpsi_neg':self.dlgnf_dpsi_neg})
        self.variables.update({'Zeff':self.Zeff})
        self.variables.update({'u_tor':self.u_tor})
        self.variables.update({'Er':self.Er})
        self.variables.update({'q':self.q})
        self.variables.update({'dlgq_dpsi':self.dlgq_dpsi})
        self.variables.update({'g':self.g})
        self.variables.update({'P':self.P})
        self.variables.update({'r_1d':self.r_1d})
        self.variables.update({'torpsi':self.torpsi})
        self.variables.update({'rg':self.rg})
        self.variables.update({'psi_tor':self.psi_tor})
        self.variables.update({'psi_rg':self.psi_rg})
        self.variables.update({'cos_err':self.cos_err})
        self.variables.update({'sin_err':self.sin_err})

        # spdata variable set
        self.variables.update({'nplot':self.nplot})
        self.variables.update({'npsi':self.npsi})
        self.variables.update({'ntheta':self.ntheta})

        self.variables.update({'R':self.R})
        self.variables.update({'Z':self.Z})
        self.variables.update({'B':self.B})
        self.variables.update({'Jacobian':self.Jacobian})
        self.variables.update({'I':self.I})
        self.variables.update({'nu':self.nu})
        self.variables.update({'delta':self.delta})

        #raise NotImplemented('equilibrium.out loader is not implemented.')

    def load_bp(self, verbose):
        """Read Equilibrium data from the ADIOS equilibrium.bp file
           
           Generates the data properties with the same name as those in the
           equilibrium file.
        """
        import adios2

        fr = adios2.open(self.filename, "r", MPI.COMM_SELF)

        names = fr.available_variables()
        self.variables = names

        for key in names.keys():
            if names[key]['SingleValue'] == 'true':
                self.__dict__[key] = fr.read(key)
            else:
                ########### to be justified ####################
                #shape = parse_shape(names[key]['Shape'])
                #start = np.zeros_like(shape)
		            #self.__dict__[key] = start
                
                self.__dict__[key] = fr.read(key)

        if verbose:
            print("Read variables: ", self.variables.keys())

    def load_h5(self, verbose):
        raise NotImplemented('equilibrium.h5 loader is not implemented.')
