""" GTC history data handler

Authors: Lei Shi
Date: Oct. 5, 2017

This module contains the general data handler for the new GTC history file.
History data analysis and visualization classes and functions should also
be kept in this module.

#All future updates should be recorded below:
#
#Author:
#Date:
#Update summary:

#######################################

Author: Wenhao Wang
Date: Aug. 27, 2018
Update summary:

Add a new function to read "*.bp" file. The code ressembles the loaders 
for "Equilibrium.bp" and "Snap.bp".

#######################################

Author: Wenhao Wang
Date: Oct. 27, 2018
Update summary:

Modify the function to read "*.bp" file. Use ADIOS2 read function, i.e. 
fr.read(), to load the multistep variables in the bp file.

"""
from __future__ import print_function
import warnings
from pprint import pprint

import numpy as np
from scipy.io.netcdf import netcdf_file

# Currently, ADIOS2 on Titan requires MPI initialization. This will give
# trouble to non-MPI calls using this module
from mpi4py import MPI
# import adios2

from ..utility import parse_shape

class gtcHistory(object):
    """main class handling GTC history data"""

    def __init__(self, fname, verbose=False):
        self.filename = fname
        self.load(verbose=verbose)

    def load(self,verbose):
        """Generic loading method, determines concrete loading scheme based on
           the extension of the filename
           parameters:
             bool verbose: print out data structure if True.
        """
        print("\n****** HISTORY PROCESSING *******\n")
        if ".out" in self.filename:
#            warnings.warn("history.out is a deprecated history format."+
#                          "Try to use new output format for new runs.")
            print("Warning: history.out is a deprecated history format."+
                  "Try to use new output format for new runs.")
            print("OLD OUTPUT file detected.")
            self.load_old(verbose=verbose)
        elif ".nc" in self.filename:
            print("NetCDF file detected.")
            self.load_nc(verbose=verbose)
        elif ".bp" in self.filename:
            print("ADIOS2 BP file detected.")
            self.load_bp(verbose=verbose)
        else:
            raise gtcHistoryException('File format not supported!' +
                                     ' Make sure you keep the file extension'+
                                     ' unchanged.')

    def load_nc(self, verbose):
        """load netCDF format GTC history file
           parameters:
             bool verbose: print out data structure if True.
        """
        history_file = netcdf_file(self.filename)
        # obtain the dimensions
        self.mpsi = history_file.dimensions['mpsi']
        # The time dimension is UNLIMITED, we get it from the length of the
        # record
        self.nt = len(history_file.variables['t'].data)
        # Equilibrium values
        # normalized poloidal flux w.r.t. separatrix flux
        self.psi_n = np.copy(history_file.variables['psi'].data)
        # poloidal flux in physics units in Maxwell
        self.psi = np.copy(history_file.variables['psi'].data) *\
                   history_file.variables['psi'].scale_factor
        # outer midplane minor radius in physics unit in cm
        self.r = np.copy(history_file.variables['r'].data) *\
                   history_file.variables['r'].scale_factor
        # equilibrium density in cm^-3
        self.ne0 = np.copy(history_file.variables['ne0'].data) *\
                   history_file.variables['ne0'].scale_factor
        # equilibrium electron temperature in eV
        self.Te0 = np.copy(history_file.variables['Te0'].data) *\
                   history_file.variables['Te0'].scale_factor
        # Time series
        # time in second
        self.t = np.copy(history_file.variables['t'].data) *\
                         history_file.variables['t'].scale_factor
        # zonal electric potential in Te0/e unit
        self.zonal_phi = np.copy(history_file.variables['zonal_phi'].data) *\
                         history_file.variables['zonal_phi'].scale_factor
        # zonal electron density perturbation in local ne0 unit
        self.zonal_ne = np.copy(history_file.variables['zonal_ne'].data)
        
#        if verbose:
#            print("Read vareables: ", self.variables.keys())

    def load_old(self, verbose):
        """Read history.out
           parameters:
             bool verbose: print out data structure if True.
        """
        lines = open(self.filename, 'r').readlines()
        ndstep = int(lines[0])   # of time steps
        nspecies = int(lines[1]) # of species: ion, electron, EP, impuries
        mpdiag = int(lines[2])   # of quantities per species: density,entropy,momentum,energy, fluxes
        nfield = int(lines[3])   # of field variables: phi, a_par, fluidne
        modes = int(lines[4])    # of modes per field: (n,m)
        mfdiag = int(lines[5])   # of quantities per field: rms, single spatial point, zonal components
        tstep = float(lines[6])  # time step size

        #######################################################################
        ### BEGINNING OF DIRTY READER STUFF, FROM UGLY GTC OUTPUT
        ######################################################################
        #if printstuff=='yes':
        #    print "# tsteps","# species", "# diagnostics", "# fields", "# modes", "# field quants", "tstep size"
        #    print ndstep, nspecies, mpdiag, nfield, modes, mfdiag, tstep
        total_len = ndstep*((mpdiag*nspecies)+(mfdiag*nfield)+(modes*nfield*2))
        if (total_len-len(lines[7:])!=0):
                total_len = len(lines[7:])
                ndstep = total_len/((mpdiag*nspecies)+(mfdiag*nfield)+(modes*nfield*2))
        ndstep = int(ndstep)
        ndstep_len = int(total_len/ndstep)
        hist_len = [(mpdiag*nspecies),(mfdiag*nfield),(modes*nfield*2)]
        history = np.zeros(total_len)
        #print(ndstep,ndstep_len)
        hist_separated = np.zeros((ndstep,ndstep_len))

        partdata = np.zeros((ndstep,hist_len[0]))
        fielddata = np.zeros((ndstep,hist_len[1]))
        modedata = np.zeros((ndstep,hist_len[2]))

        parthist = np.zeros((ndstep,mpdiag,nspecies))
        fieldhist = np.zeros((ndstep,mfdiag,nfield))
        modehist = np.zeros((ndstep,2,modes,nfield))
        for i in range(0,total_len):
            history[i] = float(lines[7+i])
        for j in range(0,ndstep):
            for i in range(0,ndstep_len):
                hist_separated[j,i] = history[(j*ndstep_len)+i]
        for it in range(0,ndstep):
            partdata[it,0:hist_len[0]] = hist_separated[it,0:hist_len[0]]
            fielddata[it,0:hist_len[1]] = hist_separated[it,hist_len[0]:(hist_len[1]+hist_len[0])]
            modedata[it,0:len(hist_separated[0,:])-hist_len[0]-hist_len[1]] = hist_separated[it,(hist_len[1]+hist_len[0]):len(hist_separated[0,:])]


        #######################################################################
        ### END OF DIRTY READER STUFF, FROM UGLY GTC OUTPUT
        ######################################################################

        ## create data structure similar to adios2 load_bp()

        self.__dict__['ndstep'] = ndstep   # of time steps
        self.__dict__['nspecies'] = nspecies # of species: ion, electron, EP, impuries
        self.__dict__['mpdiag'] = mpdiag   # of quantities per species: density,entropy,momentum,energy, fluxes
        self.__dict__['nfield'] = nfield   # of field variables: phi, a_par, fluidne
        self.__dict__['modes'] = modes    # of modes per field: (n,m)
        self.__dict__['mfdiag'] = mfdiag   # of quantities per field: rms, single spatial point, zonal components
        self.__dict__['tstep*ndiag'] = tstep  # time step size
        self.__dict__['partdata'] = np.transpose(parthist,(0,2,1))  #partdata(ndstep,nspecies,mpdiag)
        self.__dict__['fieldmode'] = np.transpose(modehist,(0,3,2,1))  #fieldmode(ndstep,nfield,modes,2)
        self.__dict__['fieldtime'] = np.transpose(fieldhist,(0,2,1))  #fieldtime(ndstep,nfield,mfdiag)

        ## update values for parthist, modehist and fieldhist. 
        ## notice that this can be done after we define the data structure.

        ## the particle array
        for it in range(0,ndstep):
            for k in range(0,nspecies):
                for j in range(0,mpdiag):
                    rightnow = ((k*mpdiag)+j)
                    parthist[it,j,k] = partdata[it,rightnow]

        ## the field array
        for it in range(0,ndstep):
            for k in range(0,nfield):
                for j in range(0,mfdiag):
                    rightnow = ((k*mfdiag)+j)
                    fieldhist[it,j,k] = fielddata[it,rightnow]

        for it in range(0,ndstep):
            for k in range(0,nfield):
                for j in range(0,modes):
                    rightnow = ((k*modes)+j)*2
                    modehist[it,0,j,k] = modedata[it,rightnow]
                    modehist[it,1,j,k] = modedata[it,rightnow+1]

        ## make a python dict to hold everything as well.
        self.variables = {}
 
        self.variables.update({'ndstep':self.ndstep})
        self.variables.update({'nspecies':self.nspecies})
        self.variables.update({'mpdiag':self.mpdiag})
        self.variables.update({'nfield':self.nfield})
        self.variables.update({'modes':self.modes})
        self.variables.update({'mfdiag':self.mfdiag})
        self.variables.update({'tstep*ndiag':self.__dict__['tstep*ndiag']})
        self.variables.update({'partdata':self.partdata})
        self.variables.update({'fieldmode':self.fieldmode})
        self.variables.update({'fieldtime':self.fieldtime})
 
        if verbose:
            print('The structure of this data is as follows ...')
            print(self.variables.keys())


### old python dict written by Lei Shi
#         # make a python dict to hold everything as well.
#         self.data = {}
# 
#         ## the particle array
#         for it in range(0,ndstep):
#             for k in range(0,nspecies):
#                 for j in range(0,mpdiag):
#                     rightnow = ((k*mpdiag)+j)
#                     parthist[it,j,k] = partdata[it,rightnow]
#         self.data.update({'particles':[]})
#         self.data['particles'] = {}
#         for k in range(0,nspecies):
#             self.data['particles'].update({'species'+str(k+1):[]})
#             self.data['particles']['species'+str(k+1)] = {}
#             for j in range(0,mpdiag):
#                 self.data['particles']['species'+str(k+1)].update({'diag'+str(j+1):[]})
#                 self.data['particles']['species'+str(k+1)]['diag'+str(j+1)] = parthist[:,j,k]
# 
#         ## the field array
#         for it in range(0,ndstep):
#             for k in range(0,nfield):
#                 for j in range(0,mfdiag):
#                     rightnow = ((k*mfdiag)+j)
#                     fieldhist[it,j,k] = fielddata[it,rightnow]
#         self.data.update({'fields':[]})
#         self.data['fields'] = {}
#         for k in range(0,nfield):
#             self.data['fields'].update({'field'+str(k+1):[]})
#             self.data['fields']['field'+str(k+1)] = {}
#             for j in range(0,mfdiag):
#                 self.data['fields']['field'+str(k+1)].update({'diag'+str(j+1):[]})
#                 self.data['fields']['field'+str(k+1)]['diag'+str(j+1)] = fieldhist[:,j,k]
# 
#         for it in range(0,ndstep):
#             for k in range(0,nfield):
#                 for j in range(0,modes):
#                     rightnow = ((k*modes)+j)*2
#                     modehist[it,0,j,k] = modedata[it,rightnow]
#                     modehist[it,1,j,k] = modedata[it,rightnow+1]
#         self.data.update({'modes':[]})
#         self.data['modes'] = {}
#         for k in range(0,modes):
#             self.data['modes'].update({'mode'+str(k+1):[]})
#             self.data['modes']['mode'+str(k+1)] = {}
#             for j in range(0,nfield):
#                 self.data['modes']['mode'+str(k+1)].update({'field'+str(j+1):{}})
#                 self.data['modes']['mode'+str(k+1)]['field'+str(j+1)].update({'real':[]})
#                 self.data['modes']['mode'+str(k+1)]['field'+str(j+1)].update({'imag':[]})
#                 self.data['modes']['mode'+str(k+1)]['field'+str(j+1)]['real'] = modehist[:,0,k,j]
#                 self.data['modes']['mode'+str(k+1)]['field'+str(j+1)]['imag'] = modehist[:,1,k,j]
# 
#         self.data['timeArray'] = np.array(range(0, ndstep))*tstep
# 
#         if verbose:
#             print('The structure of this data is as follows ...')
#             pprint(self.data, indent=4 , depth=2)

    def load_bp(self, verbose):
        """Read Snapshot data from the ADIOS snap.bp file

           Generates the data properties with the same name as those in the
           snapshot file.
           parameters:
             bool verbose: print out data structure if True.
        """
        import adios2

        fr = adios2.open(self.filename, "r", MPI.COMM_SELF)

        names = fr.available_variables()

#        assert 'mpsi' in names.keys()
#        time_count = int(names['mpsi']['AvailableStepsCount'])
        assert 'fieldmode' in names.keys()
        time_count = int(names['fieldmode']['AvailableStepsCount'])
        # print('\ntime_count = ',time_count,'\n')
        
        for key in names.keys():
            if names[key]['SingleValue'] == 'true':
                self.__dict__[key] = fr.read(key)
            else:
                shape = parse_shape(names[key]['Shape'])
                # print(shape)
                start = np.zeros_like(shape)
                tcount_tmp = int(names[key]['AvailableStepsCount'])
                self.__dict__[key] = fr.read(key, start, shape, 0, tcount_tmp)
        
        self.ndstep = time_count
        if time_count!=self.ndstep:
            self.ndstep = time_count
            print('\n\nWarning: the simulation might be interrupted, so that the \
actual timestep number is not consistant with the prescribed \
number in gtc.in. The "ndstep" value is reset to the real step number.\n\n')

        self.variables = names
        self.variables.update({'ndstep':self.ndstep})

        if verbose:
            print("Read variables: ", self.variables.keys())

        # print('\n\n\n',self.variables['partdata'],'\n\n\n')
