# -*- coding: utf-8 -*-
"""
GTC Visualization Module
=========================

This module provides plotting functions for typical GTC diagnostics.

Current main plots include

1. equilibrium plots

    1. 1D Radial Profiles
    2. 2D mesh quantities

2. history plots

    1. Mode history
    2. Particle quantities history
    
3. Snapshot plots

    1. Particle diagnostics
    2. field diagnostics

Main Components
----------------

GTC Data Loader
+++++++++++++++

A GTC output data loader object is required to feed the plotting functions with
the needed data. 

The data loader should contain properties with predefined names. An updated
variable name list needs to be maintained in the data loader module.

Plotting functions
+++++++++++++++++++

Three plotting classes corresponding to Equilibrium, History, and Snapshot are
provided. Each is initialized with a corresponding GTC Data Loader object, and 
has a set of plotting methods.

Created on Tue May 15 20:50:50 2018

@author: Lei Shi

#######################################

Author: Wenhao Wang
Date: Oct. 27, 2018
Update summary:

Start the basic construction of data1d.bp visualizer. The following functions are implemented:
None

"""

from abc import ABCMeta, abstractmethod, abstractproperty

## Add the following two lines to avoid "no module named tkinter" error.
#import matplotlib
#matplotlib.use('agg')

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import StrMethodFormatter
import tkinter as tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

from ..loader import gtcData1d
#from snapshot import gtcSnapshot
#from phi import gtcPhi
#from radialtime import gtcRadialTime


class Visualizer(object):
    """ Abstract class for all data visualizers"""
    __metaclass__ = ABCMeta
    
    @abstractmethod
    def __expr__(self):
        """Return a string containing self-explanatory information"""
        pass
    
    @abstractproperty
    def info(self):
        """self-explanatory information"""
        pass


class Data1dVisualizer(Visualizer):
    """Visualizer for GTC data1d data."""

    def __init__(self, data1d_loader, selfi):
        self._data_file = data1d_loader.filename
        self._data = data1d_loader
        self.nstart = 0
        self.nend = self._data.ndstep-1
        self.m0 = 0
        #print(self._data.ndstep)
        self.m1 = self._data.__dict__["mpsi+1"] -1
        #cutoff value
        self.cutoff = 1.0
        #print(self._data.data1di.shape)

        self.plot_mode = 0
        #To refresh the single plot window every time, use the following command with $ self._fig.clear() command in each plot function.
        #self._fig = plt.figure() #refresh the single window for each plot

    ### the master window for the plots
        self.top_win = selfi

    ### initialize plot window, fig and canvas object
        self.new_win = tk.Toplevel(self.top_win)
        self.new_win.title("Data1d Refreshing window")
        self._fig = plt.figure("Data1d Refreshing Window")
        self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)

    def _gene_plot(self, fig_name, plot_mode = 0):
#        print('plot_mode==',plot_mode)
        if plot_mode == 1:
        ### create new fig, plot window and canvas object for every plot
            self._fig = plt.figure(fig_name)  #start a new window for this plot
            self._fig.clear()
            plt.suptitle(fig_name)

            self.new_win = tk.Toplevel(self.top_win)
            self.new_win.title(fig_name)            
            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

        elif plot_mode == 0:
        ### use the previous fig, plot window and canvas object, and only clear the contents.
#            self._fig = plt.figure("Data1d Refreshing Window")
            self._fig.clear()  #refresh the existed window for this plot
            plt.suptitle(fig_name)

            self.new_win.title("Data1d Refreshing Window")
#            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

    def _canvas_update(self,canv_active):
        canv_active.get_tk_widget().pack_forget()
        canv_active.draw()
        canv_active.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_single(self, fig_name, x, y):
        """ Single panel 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        self._ax1.plot(x, y)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_double_row(self, fig_name, x1, y1, x2, y2, sharex=False):
        """ Double row panels 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(2,1,1)
        #whether the two subplots share their x axes, search online for "matplotlib add_subplot sharex"
        if sharex:
            self._ax2 = self._fig.add_subplot(2, 1, 2, sharex=self._ax1)
        else:
            self._ax2 = self._fig.add_subplot(2, 1, 2)
        self._ax1.plot(x1, y1)
        self._ax2.plot(x2, y2)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _pcolormesh(self, fig_name, x, y, f):
        """ Contour plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        #add on top: from matplotlib.ticker import MaxNLocator
        #levels = MaxNLocator(nbins=32).tick_values(f.min(),f.max())
        self._ax1.contourf(x, y, f, 13, cmap='seismic')
        #self._ax1.pcolormesh(x, y, f)
        self._ax1.set_aspect(1)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _rtime2d(self, fig_name, f):
        """ rtime2d plot"""

        # x, y range
        m_0 = int(self.m0)
        m_1 = int(self.m1)
        n_start = int(self.nstart)
        n_end = int(self.nend)
        cutoff = self.cutoff

        #x is the time range
        #y is the mpsi range
        x = range(n_start,n_end+1)
        y = range(m_0,m_1+1)
        #x = range(n_end-n_start+1)
        #y = range(m_1-m_0+1)

        # f range
        ff = np.transpose(f[n_start:n_end+1,m_0:m_1+1])
        #print(f.shape)
        #ff = f[n_start:n_end+1,m_0:m_1+1]
        #print(ff.shape)

        zmax = np.max(f)
        zmin = np.min(f)
        #print(zmin,zmax)
        zmax = cutoff * np.max([np.abs(zmax),np.abs(zmin)])
        #zmax = (0.5-cutoff)*(zmax-zmin)
        zmin = -zmax
        print(zmin,zmax)

        # contour plot
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        self._ax1.contourf(x, y, ff, 13, vmin=zmin, vmax=zmax, cmap='seismic')
        #self._ax1.pcolormesh(x, y, ff, vmin=zmin, vmax=zmax)
        #self._ax1.pcolormesh(y, x, ff)

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    ## test functions


    # Example of single-panel plot
    def plot_Er(self):
        """Plot radial electric field"""
        self._plot1D_single("Er Plot", self._data.r_1d, self._data.Er)
        self._ax1.set_xlabel("torpsi")
        self._ax1.set_ylabel("Er")
        self._canvas_update(self.canvas)

    def print_test(self):
        """test input data"""
        print(self._data.mpsi)

#############

    def contour_ion_part_flux(self):
        """Plot thermal ion particle flux"""
        self._rtime2d("Thermal ion particle flux",self._data.data1di[:,0,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("mpsi")
        self._canvas_update(self.canvas)

    def contour_ion_ene_flux(self):
        """Plot ion energy flux"""
        self._rtime2d("Ion energy flux",self._data.data1di[:,1,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("mpsi")
        self._canvas_update(self.canvas)

    def contour_e_part_flux(self):
        """Plot electron particle flux"""
        self._rtime2d("Electron particle flux",self._data.data1de[:,0,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("mpsi")
        self._canvas_update(self.canvas)

    def contour_e_ene_flux(self):
        """Plot electron energy flux"""
        self._rtime2d("Electron energy flux",self._data.data1de[:,1,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("mpsi")
        self._canvas_update(self.canvas)

    def contour_ep_part_flux(self):
        """Plot EP particle flux"""
        self._rtime2d("EP particle flux",self._data.data1df[:,0,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("mpsi")
        self._canvas_update(self.canvas)

    def contour_ep_ene_flux(self):
        """Plot EP energy flux"""
        self._rtime2d("EP energy flux",self._data.data1df[:,1,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("mpsi")
        self._canvas_update(self.canvas)

    def contour_zonal_flow(self):
        """Plot zonal flow"""
        self._rtime2d("Zonal flow",self._data.field00[:,0,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("zonal flow")
        self._canvas_update(self.canvas)

    def contour_phi_rms(self):
        """Plot phi_rms"""
        self._rtime2d("Phi_rms",self._data.fieldrms[:,0,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("phi_rms")
        self._canvas_update(self.canvas)

    def contour_zonal_i(self):
        """Plot zonal current"""
        self._rtime2d("Zonal current",self._data.field00[:,1,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("zonal current")
        self._canvas_update(self.canvas)

    def contour_apara_rms(self):
        """Plot apara_rms"""
        self._rtime2d("Apara_rms",self._data.fieldrms[:,1,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("apara_rms")
        self._canvas_update(self.canvas)

    def contour_zonal_fluid(self):
        """Plot zonal fluidne"""
        self._rtime2d("Zonal fluidne",self._data.field00[:,2,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("fluidne")
        self._canvas_update(self.canvas)

    def contour_fluid_rms(self):
        """Plot fluidne_rms"""
        self._rtime2d("Fluidne_rms",self._data.fieldrms[:,2,:])
        self._ax1.set_xlabel("time")
        self._ax1.set_ylabel("fluidne_rms")
        self._canvas_update(self.canvas)

