# -*- coding: utf-8 -*-
"""
GTC Visualization Module
=========================

This module provides plotting functions for typical GTC diagnostics.

Current main plots include

1. equilibrium plots

    1. 1D Radial Profiles
    2. 2D mesh quantities

2. history plots

    1. Mode history
    2. Particle quantities history
    
3. Snapshot plots

    1. Particle diagnostics
    2. field diagnostics

Main Components
----------------

GTC Data Loader
+++++++++++++++

A GTC output data loader object is required to feed the plotting functions with
the needed data. 

The data loader should contain properties with predefined names. An updated
variable name list needs to be maintained in the data loader module.

Plotting functions
+++++++++++++++++++

Three plotting classes corresponding to Equilibrium, History, and Snapshot are
provided. Each is initialized with a corresponding GTC Data Loader object, and 
has a set of plotting methods.

Created on Tue May 15 20:50:50 2018

@author: Lei Shi

#######################################

Author: Wenhao Wang
Date: Oct. 13, 2018
Update summary:

Finish the basic construction of snapshot.bp visualizer. The following functions are implemented:
...

#######################################

Author: Wenhao Wang
Date: Oct. 27, 2018
Update summary:

Modify the code according to the data structure of the newest bp file.

"""

from abc import ABCMeta, abstractmethod, abstractproperty

## Add the following two lines to avoid "no module named tkinter" error.
#import matplotlib
#matplotlib.use('agg')

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import StrMethodFormatter
import tkinter as tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from os.path import split as path_split

from ..loader import gtcHistory, gtcEquilibrium


class Visualizer(object):
    """ Abstract class for all data visualizers"""
    __metaclass__ = ABCMeta
    
    @abstractmethod
    def __expr__(self):
        """Return a string containing self-explanatory information"""
        pass
    
    @abstractproperty
    def info(self):
        """self-explanatory information"""
        pass


class Data3dVisualizer(Visualizer):
    """Visualizer for GTC data3d data."""

    def __init__(self, data3d_loader, selfi):
        self._data_file = data3d_loader.filename
        self._data = data3d_loader
        #timestep number for the data3d
        self._nstep = 0 # index of the data3d for the arrays
        self._realnstep = 0 # actual data3d #, i.e. self._data.step_id
        #maximum temperature
#        self.tmax = 1.0 ##########should be imported from bp files.
#        #cutoff value
#        self.cutoff = 1.0
#        #mode range
##        print(self._data.mtgrid)
#        m_tgrid = self._data.mtgrid
#        m_toroidal = self._data.mtoroidal
#        self.mmode = int(m_tgrid/5)
#        self.pmode = int(m_toroidal/5)
#        #self.mdmin = 0
#        #self.mdmax = 1

        self.plot_mode = 0
        #To refresh the single plot window every time, use the following command with $ self._fig.clear() command in each plot function.
        #self._fig = plt.figure() #refresh the single window for each plot

    ### the master window for the plots
        self.top_win = selfi

    ### initialize plot window, fig and canvas object
        self.new_win = tk.Toplevel(self.top_win)
        self.new_win.title("Data3d Refreshing window")
        self._fig = plt.figure("Data3d Refreshing Window")
        self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)

    def _gene_plot(self, fig_name, plot_mode = 0):
#        print('plot_mode==',plot_mode)
        if plot_mode == 1:
        ### create new fig, plot window and canvas object for every plot
            self._fig = plt.figure(fig_name)  #start a new window for this plot
            self._fig.clear()
            plt.suptitle(fig_name)

            self.new_win = tk.Toplevel(self.top_win)
            self.new_win.title(fig_name)            
            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

        elif plot_mode == 0:
        ### use the previous fig, plot window and canvas object, and only clear the contents.
#            self._fig = plt.figure("Data3d Refreshing Window")
            self._fig.clear()  #refresh the existed window for this plot
            plt.suptitle(fig_name)

            self.new_win.title("Data3d Refreshing Window")
#            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

    def _canvas_update(self,canv_active):
        canv_active.get_tk_widget().pack_forget()
        canv_active.draw()
        canv_active.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_single(self, fig_name, x, y):
        """ Single panel 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        self._ax1.plot(x, y)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_double_row(self, fig_name, x1, y1, x2, y2, sharex=False):
        """ Double row panels 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(2,1,1)
        #whether the two subplots share their x axes, search online for "matplotlib add_subplot sharex"
        if sharex:
            self._ax2 = self._fig.add_subplot(2, 1, 2, sharex=self._ax1)
        else:
            self._ax2 = self._fig.add_subplot(2, 1, 2)
        self._ax1.plot(x1, y1)
        self._ax2.plot(x2, y2)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _pcolormesh(self, fig_name, x, y, f):
        """ Contour plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        #add on top: from matplotlib.ticker import MaxNLocator
        #levels = MaxNLocator(nbins=32).tick_values(f.min(),f.max())
        self._ax1.contourf(x, y, f, 13, cmap='seismic')
        #self._ax1.pcolormesh(x, y, f)
        self._ax1.set_aspect(1)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _write_arrays(self,params,fid,istep=0):
        """function to generate *.out files for visit plots"""

        #path, ftmp = path_split(self._data_file)
        #print("data3d path=",path)
        #print("data3d file=",ftmp)

#        var1 = np.array([[0,1,2],[3,4,5]])
#        var2 = np.array([[0,1,2],[3,4,5]])
#        print('var1.shape=',np.shape(var1))
#        var1_flat = np.ndarray.flatten(var1)
#        var2_flat = np.ndarray.flatten(var2)
#        var1_flat = np.expand_dims(var1_flat, axis=1)
#        var2_flat = np.expand_dims(var2_flat, axis=1)
#        var_tot = np.append(var1_flat,var2_flat,axis=1)
#        print("var_tot.shape=",np.shape(var_tot))
#        np.savetxt('test.tex',var_tot,fmt="%1.4e")

        #print( params[0],"shape =",np.shape(self._data.__dict__[params[0]]) )
        #print( params[0],"size =",np.size(self._data.__dict__[params[0]]) )
        farraytmp = self._data.__dict__[params[0]][istep,:,:]
        #print( "farraytmp.shape =",np.shape(farraytmp) )
        farray = np.zeros( (np.size(farraytmp),1) )
        #print("farray.shape = ",np.shape(farray))
        for key in params:
            vartmp = self._data.__dict__[key][istep,:,:]
            var_flat = np.ndarray.flatten(vartmp)
            var_flat = np.expand_dims(var_flat,axis=1)
            #print(key,"shape = ",np.shape(var_flat))
            farray = np.append(farray, var_flat, axis=1)
        #print('farray.shape =',np.shape(farray))
        farray = np.delete(farray,0,1)
        #print('farray.shape =',np.shape(farray))
        
        #fid = open(path+"/"+fname,"w") 
        #fid.write("Hello World!\n")
        np.savetxt(fid,farray,fmt="%11.4e")


    def _gene_silo(self,params):
        """function to generate *.silo files for visit plots"""
        




    ## test functions


    # Example of single-panel plot
    def plot_Er(self):
        """Plot radial electric field"""
        self._plot1D_single("Er Plot", self._data.r_1d, self._data.Er)
        self._ax1.set_xlabel("torpsi")
        self._ax1.set_ylabel("Er")
        self._canvas_update(self.canvas)

    def print_test(self):
        """test input data"""
        print(self._data.mpsi)

#############

    def gene_out(self):

        path, ftmp = path_split(self._data_file)
        #print("data3d path=",path)
        #print("data3d file=",ftmp)

        mgrid = int(self._data.mgrid)
        mtoroidal = int(self._data.toroidal_root_comm_size)
        ## TODO: we should output mtheta in bp files 
        mtheta_surf = self._data.mtheta_surf
        step_id = np.ndarray.flatten(self._data.step_id)

        print("\n****** start generating .out files *******\n")

        ### output "bmesh_vol.out"
         
        fid = open(path+"/phi_dir/bmesh_vol.out","w") 
        fid.write("%6i %6i\n"%(mgrid,mtoroidal))
        
        params = ["b","x","y","z"]
        self._write_arrays(params,fid)

        fid.close()
        print("\nSuccess: generate bmesh_vol.out files for data3d.\n")

        ### output "phi_vol_*.out"

        print("\nStarting output phi_vol data for step_id: ",step_id)
        tcount = 0
        for tid in step_id:
            fname = '/phi_dir/phi_vol_' + str(tid).zfill(7) + '.out'
            
            fid = open(path+fname,"w") 
            
            params = ["phi"]
            self._write_arrays(params,fid,tcount)
    
            fid.close()
            tcount += 1
        print("Success: generate phi_vol_*.out files for data3d.\n")

        ### output "bmesh_surf.out"
        
        fid = open(path+"/phi_dir/bmesh_surf.out","w") 
        fid.write("%6i%6i\n"%(mtheta_surf,mtoroidal))
        
        params = ["b_surf","x_surf","y_surf","z_surf"]
        self._write_arrays(params,fid)

        fid.close()
        print("\nSuccess: generate bmesh_surf.out files for data3d.\n")

        ### output "phi_surf_*.out"

        print("\nStarting output phi_surf data for step_id: ",step_id)
        tcount = 0
        for tid in step_id:
            fname = '/phi_dir/phi_surf_' + str(tid).zfill(7) + '.out'
            
            fid = open(path+fname,"w") 
            
            params = ["phi_surf"]
            self._write_arrays(params,fid,tcount)
    
            fid.close()
            tcount += 1
        print("Success: generate phi_surf_*.out files for data3d.\n")
        print("\n****** All files are generated *******\n")

    def gene_silo(self):
        params = "input for silo"
        self._gene_silo(params)
        print("\nSuccess: generate *.silo files for data3d.\n")

    def contour_poloidal_3(self):
        """contour plot of poloidal_3"""
        n_t = self._nstep
        #n_f = self._data.nfield
        x = self._data.poloidata[n_t, self._data.nfield, :, :]
        y = self._data.poloidata[n_t, self._data.nfield+1, :, :]
        f = self._data.poloidata[n_t, 2, :, :]
        # print(x.shape, y.shape, f.shape)
        # print(self._data.nfield)
        # print(self._data.poloidata.shape)

        self._pcolormesh("Poloidal_3", x, y, f)
        # self._pcolormesh("Poloidal_1", x[0,:,:], y[0,:,:], f[0,:,:])
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        self._ax1.set_title("Poloidal")
        self._canvas_update(self.canvas)


