# -*- coding: utf-8 -*-
"""
GTC Visualization Module
=========================

This module provides plotting functions for typical GTC diagnostics.

Current main plots include

1. equilibrium plots

    1. 1D Radial Profiles
    2. 2D mesh quantities

2. history plots

    1. Mode history
    2. Particle quantities history
    
3. Snapshot plots

    1. Particle diagnostics
    2. field diagnostics

Main Components
----------------

GTC Data Loader
+++++++++++++++

A GTC output data loader object is required to feed the plotting functions with
the needed data. 

The data loader should contain properties with predefined names. An updated
variable name list needs to be maintained in the data loader module.

Plotting functions
+++++++++++++++++++

Three plotting classes corresponding to Equilibrium, History, and Snapshot are
provided. Each is initialized with a corresponding GTC Data Loader object, and 
has a set of plotting methods.

Created on Tue May 15 20:50:50 2018

@author: Lei Shi

#######################################

Author: Wenhao Wang, Jingchun Li
Date: Sept. 27, 2018
Update summary:

Finish the basic construction of snapshot.bp visualizer. The following functions are implemented:
...

#######################################

Author: Wenhao Wang
Date: Nov. 09, 2018
Update summary:

1. Correct some misused parameters, according to idl source code.
For example:
    (a) In plot_Te():
    self._plot1D_double_row("Te plot", self._data.psi/max(self._data.psi), self._data.dlgTe_dpsi_neg, self._data.psi/max(self._data.psi), self._data.Te0, sharex=True)
should be
    self._plot1D_double_row("Te plot", self._data.r_1d/self._data.r_1d[-1], self._data.dlgTe_dpsi_neg, self._data.psi, self._data.Te0)

2. Update functions:
def plot_Jacobian_theta(self)
def plot_Jacobian_radi(self)

"""

from abc import ABCMeta, abstractmethod, abstractproperty

## Add the following two lines to avoid "no module named tkinter" error.
#import matplotlib
#matplotlib.use('agg')

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import StrMethodFormatter
import tkinter as tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

from ..loader import gtcEquilibrium
#from snapshot import gtcSnapshot
#from phi import gtcPhi
#from radialtime import gtcRadialTime


class Visualizer(object):
    """ Abstract class for all data visualizers"""
    __metaclass__ = ABCMeta
    
    @abstractmethod
    def __expr__(self):
        """Return a string containing self-explanatory information"""
        pass
    
    @abstractproperty
    def info(self):
        """self-explanatory information"""
        pass


class EquilibriumVisualizer(Visualizer):
    """ Visualizer for GTC Equilibrium data. """
    def __init__(self, equilibrium_loader, selfi):
        self._data_file = equilibrium_loader.filename
        self._data = equilibrium_loader
        #grid number of psi and index range
        self.m0 = 0
        self.m1 = int(equilibrium_loader.lsp-1)
        self.idset = range(self.m0,self.m1+1)
        #
        self.isp = int(equilibrium_loader.npsi/2)

        self.plot_mode = 0
        #To refresh the single plot window every time, use the following command with $ self._fig.clear() command in each plot function.
        #self._fig = plt.figure() #refresh the single window for each plot

    ### the master window for the plots
        self.top_win = selfi

    ### initialize plot window, fig and canvas object
        self.new_win = tk.Toplevel(self.top_win)
        self.new_win.title("Equilibrium Refreshing window")
        self._fig = plt.figure("Equilibrium Refreshing Window")
        self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)

    def _gene_plot(self, fig_name, plot_mode = 0):
#        print('plot_mode==',plot_mode)
        if plot_mode == 1:
        ### create new fig, plot window and canvas object for every plot
            self._fig = plt.figure(fig_name)  #start a new window for this plot
            self._fig.clear()
            plt.suptitle(fig_name)

            self.new_win = tk.Toplevel(self.top_win)
            self.new_win.title(fig_name)            
            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

        elif plot_mode == 0:
        ### use the previous fig, plot window and canvas object, and only clear the contents.
#            self._fig = plt.figure("Equilibrium Refreshing Window")
            self._fig.clear()  #refresh the existed window for this plot
            plt.suptitle(fig_name)

            self.new_win.title("Equilibrium Refreshing Window")
#            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

    def _canvas_update(self,canv_active):
        canv_active.get_tk_widget().pack_forget()
        canv_active.draw()
        canv_active.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_single(self, fig_name, x, y):
        """ Single panel 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        self._ax1.plot(x, y)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_double_row(self, fig_name, x1, y1, x2, y2, sharex=False):
        """ Double row panels 1D plot (x, y)"""
#        print(self.plot_mode)
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(2,1,1)
        #whether the two subplots share their x axes, search online for "matplotlib add_subplot sharex"
        if sharex:
            self._ax2 = self._fig.add_subplot(2, 1, 2, sharex=self._ax1)
        else:
            self._ax2 = self._fig.add_subplot(2, 1, 2)
        self._ax1.plot(x1, y1)
        self._ax2.plot(x2, y2)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _pcolormesh(self, fig_name, x, y, f):
        """ Contour plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        #add on top: from matplotlib.ticker import MaxNLocator
        #levels = MaxNLocator(nbins=32).tick_values(f.min(),f.max())
        self._ax1.contourf(x, y, f, 32)
        #self._ax1.pcolormesh(x, y, f)
        self._ax1.set_aspect(1)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

## test functions

    # Example of single-panel plot
    def plot_Er(self):
        """Plot radial electric field"""
        self._plot1D_single("Er Plot", self._data.psi[self.idset],
                            self._data.Er[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax1.set_ylabel("Er")
        self._canvas_update(self.canvas)

#######################################
# New functions 
#######################################

    def print_test(self):
        """test input data"""
        print(self._data.torpsi)
        print(self.torpsi)
        
    def _plot1D_quadra(self, fig_name, x1, y1, x2, y2, x3, y3, x4, y4, sharex=False):
        """ Four panels 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(2,2,1)
        self._ax2 = self._fig.add_subplot(2,2,2)
        self._ax3 = self._fig.add_subplot(2,2,3)
        self._ax4 = self._fig.add_subplot(2,2,4)
        
        self._ax1.plot(x1, y1)
        self._ax2.plot(x2, y2)
        self._ax3.plot(x3, y3)
        self._ax4.plot(x4, y4)
        
        #Change the interspaces of subplots
        #Method 1:
        #plt.tight_layout()
        #Method 2:
        #left  = 0.5  # the left side of the subplots of the figure
        #right = 0.9    # the right side of the subplots of the figure
        #bottom = 0.1   # the bottom of the subplots of the figure
        #top = 0.9      # the top of the subplots of the figure
        #wspace = 0.2   # the amount of width reserved for blank space between subplots
        #hspace = 0.2   # the amount of height reserved for white space between subplots
        #plt.subplots_adjust(left, bottom, right, top, wspace, hspace)
        
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)
       
    # Example of quadra-panel plot
    def test_plot_qPanels(self):
        """Plot Te0 and -d(log Te)/torpsi"""
        self._plot1D_quadra("Quadra plot", self._data.torpsi, self._data.sin_err, self._data.torpsi, self._data.q, self._data.torpsi, self._data.sin_err, self._data.torpsi, self._data.q)
        self._ax1.set_xlabel("x1")
        self._ax1.set_ylabel("y1")
        self._ax2.set_xlabel("torpsi")
        self._ax2.set_ylabel("$T_{e0}$")
        self._ax3.set_xlabel("x3")
        self._ax3.set_ylabel("y3")
        self._ax4.set_xlabel("x4")
        self._ax4.set_ylabel("y4")
        self._canvas_update(self.canvas)

    # spline mesh
    def _spline_mesh(self, fig_name, x, y):
        """ Spline mesh plot in (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        dims = x.shape
        for i_theta in range(0,dims[0]):
            self._ax1.plot(x[i_theta,0:dims[1]],y[i_theta,0:dims[1]],"b-",linewidth=1)
        for i_psi in range(0,dims[1]):
            self._ax1.plot(x[0:dims[0],i_psi],y[0:dims[0],i_psi],"b-",linewidth=1)
        
        self._ax1.set_aspect(1)
 
        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

## construction process:
    # spline mesh
#    def _spline_mesh(self, fig_name, x, y):
#        """ spline mesh plot in (x, y)"""
#        self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
#        self._ax1 = self._fig.add_subplot(1,1,1)
#        dims = x.shape
#        psi=0
     #   for psi in range(0,dims[0]):
#        self._ax1.plot(x[1,0:dims[1]],y[1,0:dims[1]])
    #    for theta in range(0,dims[1]):
    #        self._ax1.plot(x[0:dims[0],theta],y[0:dims[0],theta])
    
    # Example of plotting the spline mesh
#    def spline_mesh(self):
#        self._spline_mesh("Spline mesh", self._data.R, self._data.Z)
#        self._ax1.set_xlabel("R")
#        self._ax1.set_ylabel("Z")
#        self._ax1.set_title("Spline mesh")

#######################################

#01 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    def plot_Majorradius(self):
        """Plot major radius"""
        self._plot1D_single("Major radius from profile data", 
                            self._data.psi[self.idset], self._data.Rmajor[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax1.set_ylabel("major R")
        self._canvas_update(self.canvas)

#02 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    def plot_Minorradius(self):
        """Plot minor radius"""
        self._plot1D_single("Minor radius from profile data", 
                            self._data.psi[self.idset], self._data.Rminor[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax1.set_ylabel("minor r")
        self._canvas_update(self.canvas)

    # Example of double-panel plot
    def plot_Te(self):
        """Plot Te0 and -d(log Te)/psi"""
        r = self._data.r_1d[self.idset]/max(self._data.r_1d)
        self._plot1D_double_row("Te plot", r, self._data.dlgTe_dpsi_neg[self.idset], 
                                self._data.psi[self.idset], self._data.Te0[self.idset])
        #self._plot1D_double_row("Te plot", self._data.r_1d, self._data.dlgTe_dpsi_neg,
        #                        self._data.psi, self._data.Te0)
    #  /max(self._data.psi)
        self._ax1.set_xlabel("$r/r_0$")
        self._ax1.set_ylabel("$-d\; ln(T_{e0})/ d\psi$")
        self._ax2.set_xlabel("psi")
        self._ax2.set_ylabel("$T_{e0}$")
        #self._ax1.set_title('-dlnTe/dpsi')
        #self._ax2.set_title('Te(psi)')
        self._canvas_update(self.canvas)

    def plot_ne(self):
        """Plot ne0 and -d(log ne)/psi"""
        r = self._data.r_1d[self.idset]/self._data.r_1d[-1]
        self._plot1D_double_row("Ne plot", r, self._data.dlgne_dpsi_neg[self.idset], 
                                self._data.psi[self.idset], self._data.ne0[self.idset])
        self._ax1.set_xlabel("$r/r_0$")
        self._ax2.set_xlabel("psi")
        self._ax1.set_ylabel("$-d\; ln(n_{e0})/ d\psi$")
        self._ax2.set_ylabel("$n_{e0}$")
        #self._ax1.set_title('-dlnne/dpsi')
        #self._ax2.set_title('ne(psi)')
        self._canvas_update(self.canvas)

    def plot_Ti(self):
        """Plot Ti0 and -d(log Ti)/psi"""
        r = self._data.r_1d[self.idset]/self._data.r_1d[-1]
        self._plot1D_double_row("Ti plot", r, self._data.dlgTi_dpsi_neg[self.idset], 
                                self._data.psi[self.idset], self._data.Ti0[self.idset])
        self._ax1.set_xlabel("$r/r_0$")
        self._ax2.set_xlabel("psi")
        self._ax1.set_ylabel("$-d\; ln(T_{i0})/ d\psi$")
        self._ax2.set_ylabel("$T_{i0}$")
        #self._ax1.set_title('-dlnTi/dpsi')
        #self._ax2.set_title('Ti(psi)')
        self._canvas_update(self.canvas)

    def plot_ni(self):
        """Plot ni and -d(log ni)/psi"""
        r = self._data.r_1d[self.idset]/self._data.r_1d[-1]
        self._plot1D_double_row("Ni plot", r, self._data.dlgni_dpsi_neg[self.idset], 
                                self._data.psi[self.idset], self._data.ni0[self.idset])
        self._ax1.set_xlabel("$r/r_0$")
        self._ax2.set_xlabel("psi")
        self._ax1.set_ylabel("$-d\; ln(n_{i0})/ d\psi$")
        self._ax2.set_ylabel("$n_{i0}$")
        #self._ax1.set_title('-dlnni/dpsi')
        #self._ax2.set_title('ni(psi)')
        self._canvas_update(self.canvas)

    def plot_Tf(self):
        """Plot Tf and -d(log Tf)/psi"""
        r = self._data.r_1d[self.idset]/self._data.r_1d[-1]
        self._plot1D_double_row("Tf plot", r, self._data.dlgtf_dpsi_neg[self.idset], 
                                self._data.psi[self.idset], self._data.Tf0[self.idset])
        self._ax1.set_xlabel("$r/r_0$")
        self._ax2.set_xlabel("psi")
        self._ax1.set_ylabel("$-d\; ln(T_{f0})/ d\psi$")
        self._ax2.set_ylabel("$T_{f0}$")
        #self._ax1.set_title('-dlnTf/dpsi')
        #self._ax2.set_title('Tf(psi)')
        self._canvas_update(self.canvas)

    def plot_nf(self):
        """Plot nf and -d(log nf)/psi"""
        r = self._data.r_1d[self.idset]/self._data.r_1d[-1]
        self._plot1D_double_row("Nf plot", r, self._data.dlgnf_dpsi_neg[self.idset], 
                                self._data.psi[self.idset], self._data.nf0[self.idset])
        self._ax1.set_xlabel("$r/r_0$")
        self._ax2.set_xlabel("psi")
        self._ax1.set_ylabel("$-d\; ln(n_{f0})/ d\psi$")
        self._ax2.set_ylabel("$n_{f0}$")
        #self._ax1.set_title('-dlnnf/dpsi')
        #self._ax2.set_title('nf(psi)')
        self._canvas_update(self.canvas)

    def plot_zeff(self):
        """Plot Zeff(psi)"""
        self._plot1D_single("Toroidal psi Plot", self._data.psi[self.idset], self._data.Zeff[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax1.set_ylabel("Zeff")
        self._canvas_update(self.canvas)

#######################################

    def plot_rotation(self):
        """Plot u_tor"""
        self._plot1D_single("Rotation plot", self._data.psi[self.idset], self._data.u_tor[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax1.set_ylabel("$u_{tor}$")
        self._canvas_update(self.canvas)


    def plot_q(self):
        """Plot q(psi) and q(r)"""
        self._plot1D_double_row("q plot", self._data.psi[self.idset], self._data.q[self.idset], 
                                self._data.r_1d[self.idset]/self._data.r_1d[-1], self._data.q[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("r")
        self._ax1.set_ylabel("$q(psi)$")
        self._ax2.set_ylabel("$q(r)$")
        self._canvas_update(self.canvas)


    def plot_s(self):
        """Plot psi(r) and -d(log q)/psi, i.e. shear(psi), s(r)"""
        self._plot1D_double_row("s plot", self._data.psi[self.idset], self._data.dlgq_dpsi[self.idset], 
                                self._data.r_1d[self.idset], self._data.psi[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("r")
        self._ax1.set_ylabel("$d(log q)/psi$")
        self._ax2.set_ylabel("$psi(r)$")
        self._canvas_update(self.canvas)


    def plot_g(self):
        """Plot current g(psi) and g(r)"""
        self._plot1D_double_row("G plot", self._data.psi[self.idset], self._data.g[self.idset], 
                                self._data.r_1d[self.idset], self._data.g[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("r")
        self._ax1.set_ylabel("$g(psi)$")
        self._ax2.set_ylabel("$g(r)$")
        self._canvas_update(self.canvas)


    def plot_p(self):
        """Plot pressure p(psi) and p(r)"""
        self._plot1D_double_row("P plot", self._data.psi[self.idset], self._data.P[self.idset], 
                                self._data.r_1d[self.idset], self._data.P[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("r")
        self._ax1.set_ylabel("$p(psi)$")
        self._ax2.set_ylabel("$p(r)$")
        self._canvas_update(self.canvas)


    def plot_tor(self):
        """Plot toroidal flux tor(psi) and tor(r)"""
        self._plot1D_double_row("Tor plot", self._data.psi[self.idset], self._data.torpsi[self.idset], 
                                self._data.r_1d[self.idset], self._data.torpsi[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("r")
        self._ax1.set_ylabel("$tor(psi)$")
        self._ax2.set_ylabel("$tor(r)$")
        self._canvas_update(self.canvas)


    def plot_r(self):
        """Plot r(psi) and r(toroidal flux)"""
        self._plot1D_double_row("r plot", self._data.psi[self.idset], self._data.r_1d[self.idset], 
                                self._data.torpsi[self.idset], self._data.r_1d[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("torpsi")
        self._ax1.set_ylabel("$r(psi)$")
        self._ax2.set_ylabel("$r(tor)$")
        self._canvas_update(self.canvas)


    def plot_rg(self):
        """Plot radial grid rg(psi) and rg(r)"""
        self._plot1D_double_row("Rg plot", self._data.psi[self.idset], self._data.rg[self.idset], 
                                self._data.r_1d[self.idset], self._data.rg[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("r")
        self._ax1.set_ylabel("$rg(psi)$")
        self._ax2.set_ylabel("$rg(r)$")
        self._canvas_update(self.canvas)

#######################################

#18 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    def plot_psit(self):
        """Plot toroidal flux and its inverse tor(psi) & psi(tor)"""
        self._plot1D_single("Tor-psi Plot", self._data.psi[self.idset], self._data.torpsi[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax1.set_ylabel("Toroidal flux")
        self._canvas_update(self.canvas)


#19 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
    def plot_psirg(self):
        """Plot radial grid and its inverse rg(psi) & psi(rg)"""
        self._plot1D_single("Rg-psi Plot", self._data.psi[self.idset], self._data.rg[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax1.set_ylabel("rg")
        self._canvas_update(self.canvas)


#20 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    def plot_qpsi(self):
        """Plot q(psi) from spdata & dtorpsi/dpsi"""
        self._plot1D_single("q(psi) Plot", self._data.psi[self.idset], self._data.q[self.idset])
        self._ax1.set_xlabel("psi")
        self._ax1.set_ylabel("q")
        self._canvas_update(self.canvas)


#21 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    def plot_psir(self):
        """Plot psi(rg) & r(rg)"""
        self._plot1D_double_row("psi_r plot", self._data.rg[self.idset], self._data.psi[self.idset], self._data.rg[self.idset], self._data.r_1d[self.idset], sharex=False)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("rg")
        self._ax1.set_ylabel("$psi$")
        self._ax2.set_ylabel("$r$")
        self._canvas_update(self.canvas)

    # Example of plotting the spline mesh
    def spline_mesh(self):
        """Plot spline mesh"""
        self._spline_mesh("Spline mesh", self._data.R, self._data.Z)
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        #self._ax1.set_title("Spline mesh")
        self._canvas_update(self.canvas)


    def plot_bfield(self):
        """contour plot of the B_field"""
        self._pcolormesh("B_field", self._data.R, self._data.Z, self._data.B)
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        #self._ax1.set_title("B_field")
        self._canvas_update(self.canvas)

    def contour_Jacobian(self):
        """contour plot of the Jacobian"""
        self._pcolormesh("Jacobian contour", self._data.R, self._data.Z, self._data.Jacobian)
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        #self._ax1.set_title("Jacobian")
        self._canvas_update(self.canvas)

    def contour_icurrent(self):
        """contour plot of I_current"""
        self._pcolormesh("I_current contour", self._data.R, self._data.Z, self._data.I)
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        self._ax1.set_title("I_current")
        self._canvas_update(self.canvas)

    def contour_zeta2phi(self):
        """contour plot of zeta2phi"""
        self._pcolormesh("Zeta2phi contour", self._data.R, self._data.Z, self._data.nu)
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        self._ax1.set_title("Zeta2phi")
        self._canvas_update(self.canvas)

#######################################

    def contour_delta_b(self):
        """contour plot of delta_b"""
        self._pcolormesh("Delta_b contour", self._data.R, self._data.Z, self._data.delta)
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        self._ax1.set_title("Delta_b")
        self._canvas_update(self.canvas)

    def _set_isp(self, isp):
        self.isp = isp

    def plot_bfield_theta(self):
        """Plot B_field(theta) at psi=isp"""
        theta_Arry = np.linspace(0,2.0*np.pi,self._data.ntheta)
        #print(theta_Arry,self._data.B[self._data.npsi-1,:])
        #print(self._data.B.shape)
        #print(self._data.ntheta, self._data.npsi)
        self._plot1D_single("B_field(theta) Plot at psi=isp", theta_Arry, self._data.B[:,self.isp])
        self._ax1.set_xlabel("theta")
        self._ax1.set_ylabel("B_field")
        self._canvas_update(self.canvas)

    def plot_Jacobian_theta(self):
        """Plot Jacobian & (gq+I)/B^2 at psi=isp"""
        theta_Arry = np.linspace(0,2.0*np.pi,self._data.ntheta)
        self._plot1D_single("Jacobian(theta) Plot at psi=isp", theta_Arry, self._data.Jacobian[:,self.isp])
        self._ax1.set_title("Jacobian & (gq+I)/B^2 at psi=isp")
        self._canvas_update(self.canvas)

    def plot_Jacobian_radi(self):
        """Plot Jacobian & (gq+I)/B^2 at theta=isp"""
        #theta_Arry = np.linspace(0,2.0*np.pi,self._data.ntheta)
        #self._plot1D_single("Jacobian(theta) Plot at psi=isp", theta_Arry, self._data.Jacobian[:,self.isp])
        #self._ax1.set_title("Jacobian & (gq+I)/B^2 at psi=isp")
        n_psi = int(self._data.npsi)
        i_sp = self.isp
        #i_sp = 1
        #J = (gq+I)/B^2
        J = np.zeros(n_psi)
        id_npsi = np.zeros(n_psi)
        mskip = int(np.round(self._data.lsp/n_psi))
        #print(n_psi,mskip)
        for i in range(n_psi):
            id_npsi[i] = int(i*mskip)
            J[i] = (self._data.g[i*mskip]*self._data.q[i*mskip]
                    +self._data.I[i_sp,i]) / (self._data.B[i_sp,i])**2
        id_npsi = np.array(id_npsi,dtype=int)
        #id_npsi.astype(int)

#        self._plot1D_double_row("Jacobian(r) & (gq+I)/B^2 at theta=isp", 
#                                self._data.r_1d[id_npsi], self._data.Jacobian[i_sp,:], 
#                                self._data.r_1d[id_npsi], J, sharex=True)
#        #self._ax1.set_xlabel("psi")
#        self._ax2.set_xlabel("r")
#        self._ax1.set_ylabel("Jacobian")
#        self._ax2.set_ylabel("$(gq+I)/B^2$")

        self._plot1D_single("Jacobian(r) & (gq+I)/B^2 at theta=isp", 
                                self._data.r_1d[id_npsi], self._data.Jacobian[i_sp,:])
        #plt.plot(self._data.r_1d[id_npsi], J, label='(gq+I)/B^2')
        plt.plot(self._data.r_1d[id_npsi], J)
        plt.legend(['Jacobian','(gq+I)/B^2'])
        self._ax1.set_xlabel("r")
        self._ax1.set_ylabel("Jacobian & (gq+I)/B^2")
        self._canvas_update(self.canvas)

    def plot_spline_err(self):
        """Plot error of spline cos and sin"""
        self._plot1D_double_row("Spline error plot", self._data.r_1d[self.idset], self._data.cos_err[self.idset], 
                                self._data.r_1d[self.idset], self._data.sin_err[self.idset], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("r")
        self._ax1.set_ylabel("Cosine error")
        self._ax2.set_ylabel("Sine error")
        self._canvas_update(self.canvas)
