# -*- coding: utf-8 -*-
"""
GTC Visualization Module
=========================

This module provides plotting functions for typical GTC diagnostics.

Current main plots include

1. equilibrium plots

    1. 1D Radial Profiles
    2. 2D mesh quantities

2. history plots

    1. Mode history
    2. Particle quantities history
    
3. Snapshot plots

    1. Particle diagnostics
    2. field diagnostics

Main Components
----------------

GTC Data Loader
+++++++++++++++

A GTC output data loader object is required to feed the plotting functions with
the needed data. 

The data loader should contain properties with predefined names. An updated
variable name list needs to be maintained in the data loader module.

Plotting functions
+++++++++++++++++++

Three plotting classes corresponding to Equilibrium, History, and Snapshot are
provided. Each is initialized with a corresponding GTC Data Loader object, and 
has a set of plotting methods.

Created on Tue May 15 20:50:50 2018

@author: Lei Shi

#######################################

Author: Wenhao Wang
Date: Oct. 27, 2018
Update summary:

Start the basic construction of data1d.bp visualizer. The following functions are implemented:
None

"""

from abc import ABCMeta, abstractmethod, abstractproperty

## Add the following two lines to avoid "no module named tkinter" error.
#import matplotlib
#matplotlib.use('agg')

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import StrMethodFormatter
import tkinter as tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

from ..loader import gtcHistory
#from snapshot import gtcSnapshot
#from phi import gtcPhi
#from radialtime import gtcRadialTime


class Visualizer(object):
    """ Abstract class for all data visualizers"""
    __metaclass__ = ABCMeta
    
    @abstractmethod
    def __expr__(self):
        """Return a string containing self-explanatory information"""
        pass
    
    @abstractproperty
    def info(self):
        """self-explanatory information"""
        pass


class HistoryVisualizer(Visualizer):
    """Visualizer for GTC history data."""

    def __init__(self, history_loader, selfi):
        self._data_file = history_loader.filename
        self._data = history_loader

        # assert 'fieldmode' in self._data.variables.keys()
        # time_count = int(self._data.variables['fieldmode']['AvailableStepsCount'])
        # if time_count != self._data.ndstep:
        #     self.tspan = range(time_count)
        # else
        #     self.tspan = range(self._data.ndstep)
        ### the timestep (i.e. ndstep) is already corrected in loader!!!
        #self.tspan = range(self._data.ndstep)
        self.nstart = 0
        self.nend = self._data.ndstep-1
        #self.nfreq = (self._data.nend-self._data.nstart)/10

        self.plot_mode = 0
        #To refresh the single plot window every time, use the following command with $ self._fig.clear() command in each plot function.
        #self._fig = plt.figure() #refresh the single window for each plot

    ### the master window for the plots
        self.top_win = selfi

    ### initialize plot window, fig and canvas object
        self.new_win = tk.Toplevel(self.top_win)
        self.new_win.title("History Refreshing window")
        self._fig = plt.figure("History Refreshing Window")
        self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)

    def _gene_plot(self, fig_name, plot_mode = 0):
#        print('plot_mode==',plot_mode)
        if plot_mode == 1:
        ### create new fig, plot window and canvas object for every plot
            self._fig = plt.figure(fig_name)  #start a new window for this plot
            self._fig.clear()
            plt.suptitle(fig_name)

            self.new_win = tk.Toplevel(self.top_win)
            self.new_win.title(fig_name)            
            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

        elif plot_mode == 0:
        ### use the previous fig, plot window and canvas object, and only clear the contents.
#            self._fig = plt.figure("History Refreshing Window")
            self._fig.clear()  #refresh the existed window for this plot
            plt.suptitle(fig_name)

            self.new_win.title("History Refreshing Window")
#            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

    def _canvas_update(self,canv_active):
        canv_active.get_tk_widget().pack_forget()
        canv_active.draw()
        canv_active.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_single(self, fig_name, x, y):
        """ Single panel 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        self._ax1.plot(x, y)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_double_row(self, fig_name, x1, y1, x2, y2, sharex=False):
        """ Double row panels 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(2,1,1)
        #whether the two subplots share their x axes, search online for "matplotlib add_subplot sharex"
        if sharex:
            self._ax2 = self._fig.add_subplot(2, 1, 2, sharex=self._ax1)
        else:
            self._ax2 = self._fig.add_subplot(2, 1, 2)
        self._ax1.plot(x1, y1)
        self._ax2.plot(x2, y2)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _pcolormesh(self, fig_name, x, y, f):
        """ Contour plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        #add on top: from matplotlib.ticker import MaxNLocator
        #levels = MaxNLocator(nbins=32).tick_values(f.min(),f.max())
        self._ax1.contourf(x, y, f, 32)
        #self._ax1.pcolormesh(x, y, f)
        self._ax1.set_aspect(1)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plotime(self, fig_name, y, title0='Plot1',title1='Plot2', sharex=True):
        """ similar to Double row panels 1D plot (x, y)"""

        n_start = int(self.nstart)
        n_end = int(self.nend)
        #x is the time range
        x = range(n_start,n_end+1)
        #print(np.shape(y))
        #print(np.shape(x))
        y0 = y[n_start:n_end+1,0]
        y1 = y[n_start:n_end+1,1]

        self._plot1D_double_row(fig_name, x, y0, x, y1, True)
        self._ax1.set_title(title0)
        self._ax2.set_title(title1)
        #plt.show()

    def _spectral(self, fig_name, y):
        """ similar to Four panels 1D plot (x, y)"""

        n_start = int(self.nstart)
        n_end = int(self.nend)
        #x is the time range
        x = range(n_start,n_end+1)

        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(2,2,1)
        self._ax2 = self._fig.add_subplot(2,2,2)
        self._ax3 = self._fig.add_subplot(2,2,3)
        self._ax4 = self._fig.add_subplot(2,2,4)
        
        # panel 1: mode history of real and imaginary components
        yr = y[n_start:n_end+1,0]
        yi = y[n_start:n_end+1,1]
        line11, = self._ax1.plot(x, yr, label = 'Re(y)')
        line12, = self._ax1.plot(x, yi, label = 'Im(y)')
        self._ax1.legend(handles=[line11, line12],loc='upper left')
        self._ax1.set_title('complex mode')

        # panel 2: mode amplitude history
        ya = np.sqrt(yr**2+yi**2)
        ypow = np.log10(ya)
        self._ax2.plot(x, ypow)
        self._ax2.set_title('logrithmic plot of mode amplitude')

        # panel 3: mode amplitude normalized by growth rate
        dt = self._data.__dict__['tstep*ndiag']
        gamma = np.log(ya[-1]/ya[0])/(x[-1]-x[0])/dt
        yr = yr/np.exp(gamma*x*dt)
        yi = yi/np.exp(gamma*x*dt)
        line31, = self._ax3.plot(x, yr, label = 'Re(y)')
        line32, = self._ax3.plot(x, yi, label = 'Im(y)')
        self._ax3.legend(handles=[line31, line32],loc='upper left')
        title = "growth rate = %.8f" % gamma
        self._ax3.set_title(title)

        # panel 4: power spectral: phi=exp[-i(omega*t+m*theta-n*zeta)]
        power = yr+yi*1j
        #dt = self._data.__dict__['tstep*ndiag']
        #nstep = self._data.ndstep
        n_step = len(power)

        fs = 1.0/dt
        power = np.fft.fft(power)
        power_mag = np.abs(power)
        #print(n_step,len(power_mag))
        P1 = power_mag[0:int(n_step/2)+1]
        P2 = power_mag[int(n_step/2)+1:n_step]
        #print(len(P1),len(P2),[int(n_step/2)+1,n_step])
        power_mag_new = np.concatenate((P2, P1), axis=None)
        #print(len(power_mag_new))
        time_num_new = np.arange(1,n_step+1,1)-(len(P2)+1)
        #print(np.shape(time_num_new),time_num_new[len(P2)])
        f_new = time_num_new*fs/n_step*np.pi*2.0 #fs = 1/dt, dt has a unit of [1/omega_cp], but frequency should be omega [omega_cp] = 2*pi*fs [omega_cp] = 2*pi/dt [omega_cp]

        self._ax4.plot(f_new, power_mag_new)
        index = np.argmax(power_mag_new) #the indices of maximum amplitude of fft.
        index = np.amin(index) #pic the first selected index for max amplitude, if there were multiple points that have the max value.
        #print(np.shape(index))
        title = "frequency = %.8f" % f_new[index]
        self._ax4.set_title(title)
        
        #Change the interspaces of subplots
        #Method 1:
        #plt.tight_layout()
        #Method 2:
        #left  = 0.5  # the left side of the subplots of the figure
        #right = 0.9    # the right side of the subplots of the figure
        #bottom = 0.1   # the bottom of the subplots of the figure
        #top = 0.9      # the top of the subplots of the figure
        #wspace = 0.2   # the amount of width reserved for blank space between subplots
        #hspace = 0.2   # the amount of height reserved for white space between subplots
        #plt.subplots_adjust(left, bottom, right, top, wspace, hspace)
        
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)
        self._canvas_update(self.canvas)


## test functions

    # # Example of single-panel plot
    # def test_plot_1(self):
    #     """Plot mode test 1.1"""
    #     tspan = self.tspan
    #     self._plot1D_single("Mode1.1 plot", tspan,
    #                         self._data.fieldmode[:,0,0,0])
    #     self._ax1.set_xlabel("step")
    #     self._ax1.set_ylabel("mode1.1")

    # def test_plot_2(self):
    #     """Plot mode test 1.2"""
    #     tspan = self.tspan
    #     self._plot1D_single("Mode1.2 plot", tspan,
    #                         self._data.fieldmode[:,1,0,0])
    #     self._ax1.set_xlabel("step")
    #     self._ax1.set_ylabel("mode1.2")

    def test_plotime(self):
        """Plot time history test"""
        #print(np.shape(self._data.fieldtime))
        self._plotime("Plotime test",self._data.fieldtime[:,0,0:1+1])
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)
    
    def test_spectral(self):
        """Plot spectral test"""
        self._spectral("Spectral test",self._data.fieldmode[:,0,0,0:1+1])

#############

    def plot_phi(self):
        """Plot phi history"""
        self._plotime("Phi",self._data.fieldtime[:,0,0:1+1],"phi(theta=zeta=0)","phip00(iflux_diag)")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_RMS_phi(self):
        """Plot phi RMS history"""
        self._plotime("RMS Plot 1",self._data.fieldtime[:,0,2:3+1],"ZF RMS","phi RMS")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_mode1_1(self):
        """Plot mode1 for species 1"""
        self._spectral("Mode1 1",self._data.fieldmode[:,0,0,0:1+1])

    def plot_mode2_1(self):
        """Plot mode2 for species 1"""
        self._spectral("Mode2 1",self._data.fieldmode[:,0,1,0:1+1])

    def plot_mode3_1(self):
        """Plot mode3 for species 1"""
        self._spectral("Mode3 1",self._data.fieldmode[:,0,2,0:1+1])

    def plot_mode4_1(self):
        """Plot mode4 for species 1"""
        self._spectral("Mode4 1",self._data.fieldmode[:,0,3,0:1+1])

    def plot_mode5_1(self):
        """Plot mode5 for species 1"""
        self._spectral("Mode5 1",self._data.fieldmode[:,0,4,0:1+1])

    def plot_mode6_1(self):
        """Plot mode6 for species 1"""
        self._spectral("Mode6 1",self._data.fieldmode[:,0,5,0:1+1])

    def plot_mode7_1(self):
        """Plot mode7 for species 1"""
        self._spectral("Mode7 1",self._data.fieldmode[:,0,6,0:1+1])

    def plot_mode8_1(self):
        """Plot mode8 for species 1"""
        self._spectral("Mode8 1",self._data.fieldmode[:,0,7,0:1+1])

#############

    def plot_apara(self):
        """Plot A_para history"""
        self._plotime("A_para",self._data.fieldtime[:,1,0:1+1],"a_par(theta=zeta=0)","a_par00(iflux_diag)")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_RMS_apara(self):
        """Plot A_para RMS history"""
        self._plotime("RMS Plot 2",self._data.fieldtime[:,1,2:3+1],"ZF RMS","a_par RMS")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_mode1_2(self):
        """Plot mode1 for species 2"""
        self._spectral("Mode1 2",self._data.fieldmode[:,1,0,0:1+1])

    def plot_mode2_2(self):
        """Plot mode2 for species 2"""
        self._spectral("Mode2 2",self._data.fieldmode[:,1,1,0:1+1])

    def plot_mode3_2(self):
        """Plot mode3 for species 2"""
        self._spectral("Mode3 2",self._data.fieldmode[:,1,2,0:1+1])

    def plot_mode4_2(self):
        """Plot mode4 for species 2"""
        self._spectral("Mode4 2",self._data.fieldmode[:,1,3,0:1+1])

    def plot_mode5_2(self):
        """Plot mode5 for species 2"""
        self._spectral("Mode5 2",self._data.fieldmode[:,1,4,0:1+1])

    def plot_mode6_2(self):
        """Plot mode6 for species 2"""
        self._spectral("Mode6 2",self._data.fieldmode[:,1,5,0:1+1])

    def plot_mode7_2(self):
        """Plot mode7 for species 2"""
        self._spectral("Mode7 2",self._data.fieldmode[:,1,6,0:1+1])

    def plot_mode8_2(self):
        """Plot mode8 for species 2"""
        self._spectral("Mode8 2",self._data.fieldmode[:,1,7,0:1+1])

#############

    def plot_fluidne(self):
        """Plot fluidne history"""
        self._plotime("Fluidne",self._data.fieldtime[:,2,0:1+1],"fluidne(theta=zeta=0)","fluidne_00(iflux_diag)")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_RMS_fluidne(self):
        """Plot zonal fluidne RMS history"""
        self._plotime("RMS Plot 3",self._data.fieldtime[:,2,2:3+1],"ZF RMS","fluidne RMS")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_mode1_3(self):
        """Plot mode1 for species 3"""
        self._spectral("Mode1 3",self._data.fieldmode[:,2,0,0:1+1])

    def plot_mode2_3(self):
        """Plot mode2 for species 3"""
        self._spectral("Mode2 3",self._data.fieldmode[:,2,1,0:1+1])

    def plot_mode3_3(self):
        """Plot mode3 for species 3"""
        self._spectral("Mode3 3",self._data.fieldmode[:,2,2,0:1+1])

    def plot_mode4_3(self):
        """Plot mode4 for species 3"""
        self._spectral("Mode4 3",self._data.fieldmode[:,2,3,0:1+1])

    def plot_mode5_3(self):
        """Plot mode5 for species 3"""
        self._spectral("Mode5 3",self._data.fieldmode[:,2,4,0:1+1])

    def plot_mode6_3(self):
        """Plot mode6 for species 3"""
        self._spectral("Mode6 3",self._data.fieldmode[:,2,5,0:1+1])

    def plot_mode7_3(self):
        """Plot mode7 for species 3"""
        self._spectral("Mode7 3",self._data.fieldmode[:,2,6,0:1+1])

    def plot_mode8_3(self):
        """Plot mode8 for species 3"""
        self._spectral("Mode8 3",self._data.fieldmode[:,2,7,0:1+1])

#############

    def plot_ion_den(self):
        """Plot ion density and entropy"""
        self._plotime("Ion density",self._data.partdata[:,0,0:1+1],"density delta_f","entropy delta_f^2")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ion_mom(self):
        """Plot ion momentum"""
        self._plotime("Ion momentum",self._data.partdata[:,0,2:3+1],"paralle flow u", "delta_u")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ion_ene(self):
        """Plot ion energy"""
        self._plotime("Ion energy",self._data.partdata[:,0,4:5+1],"energy E-1.5","delta_E")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ion_mom_flux(self):
        """Plot ion total particle and toroidal momentum flux"""
        self._plotime("Ion flux",self._data.partdata[:,0,6:7+1],"particle flux","toroidal momentum flux")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ion_ene_flux(self):
        """Plot ion energy flux, out of boundary particle ratio"""
        self._plotime("Ion flux",self._data.partdata[:,0,8:9+1],"energy flux","out of bounds ratio")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

#############

    def plot_ele_den(self):
        """Plot electron density and entropy"""
        self._plotime("Electron density",self._data.partdata[:,1,0:1+1],"density delta_f","entropy delta_f^2")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ele_mom(self):
        """Plot electron momentum"""
        self._plotime("Electron momentum",self._data.partdata[:,1,2:3+1],"paralle flow u", "delta_u")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ele_ene(self):
        """Plot electron energy"""
        self._plotime("Electron energy",self._data.partdata[:,1,4:5+1],"energy E-1.5","delta_E")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ele_mom_flux(self):
        """Plot electron total particle and toroidal momentum flux"""
        self._plotime("Electron flux",self._data.partdata[:,1,6:7+1],"particle flux","toroidal momentum flux")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ele_ene_flux(self):
        """Plot electron energy flux, out of boundary particle ratio"""
        self._plotime("Electron flux",self._data.partdata[:,1,8:9+1],"energy flux","num out bounds / num")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

#############

    def plot_ep_den(self):
        """Plot EP density and entropy"""
        self._plotime("EP density",self._data.partdata[:,2,0:1+1],"density delta_f","entropy delta_f^2")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ep_mom(self):
        """Plot EP momentum"""
        self._plotime("EP momentum",self._data.partdata[:,2,2:3+1],"paralle flow u", "delta_u")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ep_ene(self):
        """Plot EP energy"""
        self._plotime("EP energy",self._data.partdata[:,2,4:5+1],"energy E-1.5","delta_E")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ep_mom_flux(self):
        """Plot EP total particle and toroidal momentum flux"""
        self._plotime("EP flux",self._data.partdata[:,2,6:7+1],"particle flux","toroidal momentum flux")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

    def plot_ep_ene_flux(self):
        """Plot EP energy flux, out of boundary particle ratio"""
        self._plotime("EP flux",self._data.partdata[:,2,8:9+1],"energy flux","num out bounds / num")
        self._ax2.set_xlabel('time')
        self._canvas_update(self.canvas)

#############





class HistoryVisualizer_Old_Version(Visualizer):
    """ Visualizer for GTC History data. """
    
    def __init__(self, gtc_history):
        assert isinstance(gtc_history, gtcHistory)
        self._data_file = gtc_history.filename
        self._data = gtc_history.data
        self._fig = plt.figure()
        
    def __expr__(self):
        return self.info
        
    def __str__(self):
        return self.info
    
    @property
    def info(self):
        info = "GTC History Visualizer\n"
        info += "Data File: {}\n".format(self._data_file)
        return info
        
    def _plot(self, ax, y, *P, **kw):
        """General time series plotting proxy function"""
        ax.plot(self._data['timeArray'], y, *P, **kw)
        ax.set_xlabel('time(step)')
        
    def plot_phi(self, mode):
        """Plot electric potential for the chosen mode index"""
        self._fig.clear()
        ax = self._fig.add_subplot(1,1,1)
        self._plot(ax, self._data['modes']['mode'+str(mode)]['field1']['real'],
                   label='real')
        self._plot(ax, self._data['modes']['mode'+str(mode)]['field1']['imag'],
                   'r--', label='imag')
        ax.legend(loc='best')
        ax.set_title('Electric potential: mode {}'.format(mode))
        
    def plot_Apara(self, mode):
        """Plot parallel vector potential for the chosen mode index"""
        self._fig.clear()
        ax = self._fig.add_subplot(1,1,1)
        self._plot(ax, self._data['modes']['mode'+str(mode)]['field2']['real'],
                   label='real')
        self._plot(ax, self._data['modes']['mode'+str(mode)]['field2']['imag'],
                   'r--', label='imag')
        ax.legend(loc='best')
        ax.set_title('Parallel vector potential: mode {}'.format(mode))
        
    def plot_density_and_entropy(self, species):
        """Plot purterbed density for the chosen species"""
        self._fig.clear()
        ax1 = self._fig.add_subplot(2,1,1)
        ax2 = self._fig.add_subplot(2,1,2, sharex=ax1)
        self._plot(ax1, self._data['particles']['species'+str(species)]['diag1'])
        ax1.yaxis.set_major_formatter(StrMethodFormatter('{x:.1e}'))
        ax1.set_title('Density and Entropy: Species {}'.format(species))
        ax1.set_ylabel('perturbed density (df)')
        
        self._plot(ax2, self._data['particles']['species'+str(species)]['diag2'])
        ax2.yaxis.set_major_formatter(StrMethodFormatter('{x:.1e}'))
        ax2.set_ylabel('Entropy (df^2)')
        
        self._fig.tight_layout()
        
    def plot_momentum(self, species):
        """Plot total and perturbed parallel momentum for the chosen species"""
        self._fig.clear()
        ax1 = self._fig.add_subplot(2,1,1)
        ax2 = self._fig.add_subplot(2,1,2, sharex=ax1)
        self._plot(ax1, self._data['particles']['species'+str(species)]['diag3'])
        ax1.yaxis.set_major_formatter(StrMethodFormatter('{x:.1e}'))
        ax1.set_title('Parallel Momentum: Species {}'.format(species))
        ax1.set_ylabel('Total')
        
        self._plot(ax2, self._data['particles']['species'+str(species)]['diag4'])
        ax2.yaxis.set_major_formatter(StrMethodFormatter('{x:.1e}'))
        ax2.set_ylabel('Perturbed')
        
    def plot_energy(self, species):
        """Plot total and perturbed energy for the chosen species"""
        self._fig.clear()
        ax1 = self._fig.add_subplot(2,1,1)
        ax2 = self._fig.add_subplot(2,1,2, sharex=ax1)
        self._plot(ax1, self._data['particles']['species'+str(species)]['diag5'])
        ax1.yaxis.set_major_formatter(StrMethodFormatter('{x:.1e}'))
        ax1.set_title('Energy: Species {}'.format(species))
        ax1.set_ylabel('Total')
        
        self._plot(ax2, self._data['particles']['species'+str(species)]['diag6'])
        ax2.yaxis.set_major_formatter(StrMethodFormatter('{x:.1e}'))
        ax2.set_ylabel('Perturbed')

