# -*- coding: utf-8 -*-
"""
GTC Visualization Module
=========================

This module provides plotting functions for typical GTC diagnostics.

Current main plots include

1. equilibrium plots

    1. 1D Radial Profiles
    2. 2D mesh quantities

2. history plots

    1. Mode history
    2. Particle quantities history
    
3. Snapshot plots

    1. Particle diagnostics
    2. field diagnostics

Main Components
----------------

GTC Data Loader
+++++++++++++++

A GTC output data loader object is required to feed the plotting functions with
the needed data. 

The data loader should contain properties with predefined names. An updated
variable name list needs to be maintained in the data loader module.

Plotting functions
+++++++++++++++++++

Three plotting classes corresponding to Equilibrium, History, and Snapshot are
provided. Each is initialized with a corresponding GTC Data Loader object, and 
has a set of plotting methods.

Created on Tue May 15 20:50:50 2018

@author: Lei Shi

#######################################

Author: Wenhao Wang
Date: Oct. 13, 2018
Update summary:

Finish the basic construction of snapshot.bp visualizer. The following functions are implemented:
...

#######################################

Author: Wenhao Wang
Date: Oct. 27, 2018
Update summary:

Modify the code according to the data structure of the newest bp file.

"""

from abc import ABCMeta, abstractmethod, abstractproperty

## Add the following two lines to avoid "no module named tkinter" error.
#import matplotlib
#matplotlib.use('agg')

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import StrMethodFormatter
import tkinter as tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

from ..loader import gtcHistory, gtcEquilibrium
#from snapshot import gtcSnapshot
#from phi import gtcPhi
#from radialtime import gtcRadialTime


class Visualizer(object):
    """ Abstract class for all data visualizers"""
    __metaclass__ = ABCMeta
    
    @abstractmethod
    def __expr__(self):
        """Return a string containing self-explanatory information"""
        pass
    
    @abstractproperty
    def info(self):
        """self-explanatory information"""
        pass


class SnapshotVisualizer(Visualizer):
    """Visualizer for GTC snapshot data."""

    def __init__(self, snapshot_loader, selfi):
        self._data_file = snapshot_loader.filename
        self._data = snapshot_loader
        #timestep number for the snapshot
        self._nstep = 0 # index of the snapshot for the arrays
        self._realnstep = 0 # actual snap #, i.e. self._data.step_id
        #maximum temperature
        self.tmax = 1.0 ##########should be imported from bp files.
        #cutoff value
        self.cutoff = 1.0
        #mode range
#        print(self._data.mtgrid)
        m_tgrid = self._data.mtgrid
        m_toroidal = self._data.mtoroidal
        self.mmode = int(m_tgrid/5)
        self.pmode = int(m_toroidal/5)
        #self.mdmin = 0
        #self.mdmax = 1

        self.plot_mode = 0
        #To refresh the single plot window every time, use the following command with $ self._fig.clear() command in each plot function.
        #self._fig = plt.figure() #refresh the single window for each plot

    ### the master window for the plots
        self.top_win = selfi

    ### initialize plot window, fig and canvas object
        self.new_win = tk.Toplevel(self.top_win)
        self.new_win.title("Snapshot Refreshing window")
        self._fig = plt.figure("Snapshot Refreshing Window")
        self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)

    def _gene_plot(self, fig_name, plot_mode = 0):
#        print('plot_mode==',plot_mode)
        if plot_mode == 1:
        ### create new fig, plot window and canvas object for every plot
            self._fig = plt.figure(fig_name)  #start a new window for this plot
            self._fig.clear()
            plt.suptitle(fig_name)

            self.new_win = tk.Toplevel(self.top_win)
            self.new_win.title(fig_name)            
            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

        elif plot_mode == 0:
        ### use the previous fig, plot window and canvas object, and only clear the contents.
#            self._fig = plt.figure("Snapshot Refreshing Window")
            self._fig.clear()  #refresh the existed window for this plot
            plt.suptitle(fig_name)

            self.new_win.title("Snapshot Refreshing Window")
#            self.canvas = FigureCanvasTkAgg(self._fig, master=self.new_win)
            self.canvas.get_tk_widget().pack_forget()

    def _canvas_update(self,canv_active):
        canv_active.get_tk_widget().pack_forget()
        canv_active.draw()
        canv_active.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_single(self, fig_name, x, y):
        """ Single panel 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        self._ax1.plot(x, y)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _plot1D_double_row(self, fig_name, x1, y1, x2, y2, sharex=False):
        """ Double row panels 1D plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(2,1,1)
        #whether the two subplots share their x axes, search online for "matplotlib add_subplot sharex"
        if sharex:
            self._ax2 = self._fig.add_subplot(2, 1, 2, sharex=self._ax1)
        else:
            self._ax2 = self._fig.add_subplot(2, 1, 2)
        self._ax1.plot(x1, y1)
        self._ax2.plot(x2, y2)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _pcolormesh(self, fig_name, x, y, f):
        """ Contour plot (x, y)"""
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        #add on top: from matplotlib.ticker import MaxNLocator
        #levels = MaxNLocator(nbins=32).tick_values(f.min(),f.max())
        self._ax1.contourf(x, y, f, 13, cmap='seismic')
        #self._ax1.pcolormesh(x, y, f)
        self._ax1.set_aspect(1)
        #plt.show()

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)

    def _cut1d(self, fig_name, f, icut):
        """ Cut_1d plot"""
        m_psi = self._data.mpsi
        m_tgrid = self._data.mtgrid
        #f = np.transpose(f)

        #radial profile
        if icut == 1:
            x = range(m_psi)
            y1 = np.zeros(m_psi)
            y2 = np.zeros(m_psi)

            for i in x:
                #y1[i] = f[0,i]
                y1[i] = f[i,0]

                for j in range(m_tgrid):
                    #y2[i] = y2[i]+f[j,i]*f[j,i]*1.0
                    y2[i] = y2[i]+f[i,j]*f[i,j]*1.0
                y2[i] = np.sqrt(y2[i]/m_tgrid)
        
        #poloidal profile
        elif icut == 2:
            x = range(m_tgrid)
            y1 = np.zeros(m_tgrid)
            y2 = np.zeros(m_tgrid)

            for j in x:
                #y1[j] = f[j,m_psi/2]
                y1[j] = f[int(m_psi/2),j]

                for i in range(m_psi):
                    #y2[j] = y2[j]+f[j,i]*f[j,i]
                    y2[j] = y2[j]+f[i,j]*f[i,j]
                y2[j] = np.sqrt(y2[j]/m_psi)
        
        self._plot1D_double_row(fig_name, x, y1, x, y2)
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("point value")
        self._ax2.set_ylabel("rms")

    def _spectrum(self, fig_name, f):
        """ Spectrum Plot"""
        m_tgrid = self._data.mtgrid
        m_toroidal = self._data.mtoroidal
        mmode = self.mmode
        pmode = self.pmode
        #cutoff = self.cutoff

        #poloidal mode
        x1 = range(mmode)
        y1 = np.zeros(mmode)

        yy = np.zeros(m_tgrid)
        for i in range(m_toroidal):
            yy = f[i,:]
            yy = np.fft.fft(yy)
            y1[0] = y1[0]+(abs(yy[0]))**2
            for j in range(1,mmode):
                y1[j] = y1[j]+(abs(yy[j]))**2+(abs(yy[m_tgrid-j]))**2
        for j in range(mmode):
            y1[j] = np.sqrt(y1[j]/m_toroidal)/m_tgrid
        
        #parallel mode
        x2 = range(pmode)
        y2 = np.zeros(pmode)

        yy = np.zeros(m_toroidal)
        for i in range(m_tgrid):
            yy = f[:,i]
            yy = np.fft.fft(yy)
            y2[0] = y2[0]+(abs(yy[0]))**2
            for j in range(1,pmode):
                y2[j] = y2[j]+(abs(yy[j]))**2+(abs(yy[m_toroidal-j]))**2
        for j in range(pmode):
            y2[j] = np.sqrt(y2[j]/m_tgrid)/m_toroidal
        
        self._plot1D_double_row(fig_name, x1, y1, x2, y2, sharex=False)
        self._ax1.set_title("Poloidal spectrum")
        self._ax2.set_title("Parallel spectrum")
    
    def _flux(self, fig_name, f):
        """ Flux Plot"""
        m_tgrid = self._data.mtgrid
        m_toroidal = self._data.mtoroidal
        cutoff = self.cutoff

        #y = range(1,m_toroidal+1)
        y = np.zeros(m_toroidal)
        for i in range(m_toroidal):
            y[i] = 2.0*np.pi*((i+1)/m_toroidal)
        #x = range(1,m_tgrid+1)
        x = np.zeros(m_tgrid)
        for i in range(m_tgrid):
            x[i] = 2.0*np.pi*((i+1)/m_tgrid)
        #y = 2.0*np.pi*((range(m_toroidal)+1)/m_toroidal)
        #x = 2.0*np.pi*((range(m_tgrid)+1)/m_tgrid)
        X, Y = np.meshgrid(x, y)
        #print(Y.shape)
        #print(Y)
        #print(y)

        zmax = np.max(f)
        zmin = np.min(f)
        #print(zmin,zmax)
        zmax = cutoff * np.max([np.abs(zmax),np.abs(zmin)])
        #zmax = (0.5-cutoff)*(zmax-zmin)
        zmin = -zmax
        print(zmin,zmax)

        # contour plot
        self._gene_plot(fig_name, self.plot_mode)
        #self._fig = plt.figure(fig_name) #start a new window for this plot
        #self._fig.clear() #refresh the single window for each plot
        self._ax1 = self._fig.add_subplot(1,1,1)
        self._ax1.contourf(Y, X, f, 13, cmap='seismic', vmin=zmin, vmax=zmax)
        self._ax1.set_aspect(1)

#        self._ax1.pcolormesh(Y, X, f, vmin=zmin, vmax=zmax)
#        self._ax1.set_aspect(1)

        #self._pcolormesh(fig_name, Y, X, f)

        #self.canvas.draw()
        #self.canvas.get_tk_widget().pack(side='top',fill='both',expand=True)


    ## test functions


    # Example of single-panel plot
    def plot_Er(self):
        """Plot radial electric field"""
        self._plot1D_single("Er Plot", self._data.r_1d, self._data.Er)
        self._ax1.set_xlabel("torpsi")
        self._ax1.set_ylabel("Er")
        self._canvas_update(self.canvas)

    def print_test(self):
        """test input data"""
        print(self._data.mpsi)

#############

    def plot_ion_rho(self):
        """Plot ion density"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("Ion density plot", x, self._data.profile[n_t,0,0,:], x, self._data.profile[n_t,0,1,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_flow_1(self):
        """Plot flow_1"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("Flow_1 plot", x, self._data.profile[n_t,0,2,:], x, self._data.profile[n_t,0,3,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_energy_1(self):
        """Plot energy_1"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("Energy_1 plot", x, self._data.profile[n_t,0,4,:], x, self._data.profile[n_t,0,5,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_pdf_energy_1(self):
        """Plot PDF in energy_1"""
        n_t = self._nstep
        tmax = self.tmax
        x = range(self._data.nvgrid)
        x = [ tmax*x_i/(self._data.nvgrid-1) for x_i in x ]
        self._plot1D_double_row("PDF_in_energy_1 plot", x, self._data.pdf[n_t,0,0,:], x, self._data.pdf[n_t,0,1,:], sharex=True)
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_pdf_pitch_angle_1(self):
        """Plot PDF in pitch angle_1"""
        n_t = self._nstep
        x = range(self._data.nvgrid)
        self._plot1D_double_row("PDF_in_pitch_angle_1 plot", x, self._data.pdf[n_t,0,2,:], x, self._data.pdf[n_t,0,3,:], sharex=True)
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

#############

    def plot_electron_rho(self):
        """Plot electron density"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("Electron density plot", x, self._data.profile[n_t,1,0,:], x, self._data.profile[n_t,1,1,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_flow_2(self):
        """Plot flow_2"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("Flow_2 plot", x, self._data.profile[n_t,1,2,:], x, self._data.profile[n_t,1,3,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_energy_2(self):
        """Plot energy_2"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("Energy_2 plot", x, self._data.profile[n_t,1,4,:], x, self._data.profile[n_t,1,5,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_pdf_energy_2(self):
        """Plot PDF in energy_2"""
        n_t = self._nstep
        tmax = self.tmax
        x = range(self._data.nvgrid)
        x = [ tmax*x_i/(self._data.nvgrid-1) for x_i in x ]
        self._plot1D_double_row("PDF_in_energy_2 plot", x, self._data.pdf[n_t,1,0,:], x, self._data.pdf[n_t,1,1,:], sharex=True)
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_pdf_pitch_angle_2(self):
        """Plot PDF in pitch angle_2"""
        n_t = self._nstep
        x = range(self._data.nvgrid)
        self._plot1D_double_row("PDF_in_pitch_angle_2 plot", x, self._data.pdf[n_t,1,2,:], x, self._data.pdf[n_t,1,3,:], sharex=True)
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

#############

    def plot_ep_rho(self):
        """Plot EP density"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("EP density plot", x, self._data.profile[n_t,2,0,:], x, self._data.profile[n_t,2,1,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_flow_3(self):
        """Plot flow_3"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("Flow_3 plot", x, self._data.profile[n_t,2,2,:], x, self._data.profile[n_t,2,3,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_energy_3(self):
        """Plot energy_3"""
        n_t = self._nstep
        x = range(self._data.mpsi)
        self._plot1D_double_row("Energy_3 plot", x, self._data.profile[n_t,2,4,:], x, self._data.profile[n_t,2,5,:], sharex=True)
       # self._ax1.set_xlabel("psi")
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_pdf_energy_3(self):
        """Plot PDF in energy_3"""
        n_t = self._nstep
        tmax = self.tmax
        x = range(self._data.nvgrid)
        x = [ tmax*x_i/(self._data.nvgrid-1) for x_i in x ]
        self._plot1D_double_row("PDF_in_energy_3 plot", x, self._data.pdf[n_t,2,0,:], x, self._data.pdf[n_t,2,1,:], sharex=True)
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

    def plot_pdf_pitch_angle_3(self):
        """Plot PDF in pitch angle_3"""
        n_t = self._nstep
        x = range(self._data.nvgrid)
        self._plot1D_double_row("PDF_in_pitch_angle_3 plot", x, self._data.pdf[n_t,2,2,:], x, self._data.pdf[n_t,2,3,:], sharex=True)
        self._ax2.set_xlabel("index")
        self._ax1.set_ylabel("full-f")
        self._ax2.set_ylabel("del-f")
        self._canvas_update(self.canvas)

#############

    def contour_phi_fluxsurf(self):
        """Phi on flux surface"""
        n_t = self._nstep
        f = self._data.fluxdata[n_t, 0, :, :]
        self._flux("Phi on flux surface", f)
        self._canvas_update(self.canvas)

    def plot_spectrum_1(self):
        """Poloidal and parallel spectra"""
        n_t = self._nstep
        f = self._data.fluxdata[n_t, 0, :, :]
        self._spectrum("Poloidal and parallel spectra 1", f)
        self._canvas_update(self.canvas)


    def contour_poloidal_1(self):
        """contour plot of poloidal_1"""
        n_t = self._nstep
        #n_f = self._data.nfield
        x = self._data.poloidata[n_t, self._data.nfield, :, :]
        y = self._data.poloidata[n_t, self._data.nfield+1, :, :]
        f = self._data.poloidata[n_t, 0, :, :]
        # print(x.shape, y.shape, f.shape)
        # print(self._data.nfield)
        # print(self._data.poloidata.shape)

        self._pcolormesh("Poloidal_1", x, y, f)
        # self._pcolormesh("Poloidal_1", x[0,:,:], y[0,:,:], f[0,:,:])
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        self._ax1.set_title("Poloidal")
        self._canvas_update(self.canvas)

    def plot_radius_field_1(self):
        """Radius profile of field and rms 1"""
        n_t = self._nstep
        f = self._data.poloidata[n_t, 0, :, :]
        self._cut1d("Radius field and rms 1", f, 1)
        self._canvas_update(self.canvas)

    def plot_poloidal_field_1(self):
        """Poloidal profile of field and rms 1"""
        n_t = self._nstep
        f = self._data.poloidata[n_t, 0, :, :]
        self._cut1d("Poloidal field and rms 1", f, 2)
        self._canvas_update(self.canvas)

#############

    def contour_apara_fluxsurf(self):
        """A_para on flux surface"""
        n_t = self._nstep
        f = self._data.fluxdata[n_t, 1, :, :]
        self._flux("A_para on flux surface", f)
        self._canvas_update(self.canvas)

    def plot_spectrum_2(self):
        """Poloidal and parallel spectra 2"""
        n_t = self._nstep
        f = self._data.fluxdata[n_t, 1, :, :]
        self._spectrum("Poloidal and parallel spectra 2", f)
        self._canvas_update(self.canvas)


    def contour_poloidal_2(self):
        """contour plot of poloidal_2"""
        n_t = self._nstep
        #n_f = self._data.nfield
        x = self._data.poloidata[n_t, self._data.nfield, :, :]
        y = self._data.poloidata[n_t, self._data.nfield+1, :, :]
        f = self._data.poloidata[n_t, 1, :, :]
        # print(x.shape, y.shape, f.shape)
        # print(self._data.nfield)
        # print(self._data.poloidata.shape)

        self._pcolormesh("Poloidal_2", x, y, f)
        # self._pcolormesh("Poloidal_1", x[0,:,:], y[0,:,:], f[0,:,:])
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        self._ax1.set_title("Poloidal")
        self._canvas_update(self.canvas)

    def plot_radius_field_2(self):
        """Radius profile of field and rms 2"""
        n_t = self._nstep
        f = self._data.poloidata[n_t, 1, :, :]
        self._cut1d("Radius field and rms 2", f, 1)
        self._canvas_update(self.canvas)

    def plot_poloidal_field_2(self):
        """Poloidal profile of field and rms 2"""
        n_t = self._nstep
        f = self._data.poloidata[n_t, 1, :, :]
        self._cut1d("Poloidal field and rms 2", f, 2)
        self._canvas_update(self.canvas)

#############

    def contour_fluidne_fluxsurf(self):
        """Fluidne on flux surface"""
        n_t = self._nstep
        f = self._data.fluxdata[n_t, 2, :, :]
        self._flux("Fluidne on flux surface", f)
        self._canvas_update(self.canvas)

    def plot_spectrum_3(self):
        """Poloidal and parallel spectra 3"""
        n_t = self._nstep
        f = self._data.fluxdata[n_t, 2, :, :]
        self._spectrum("Poloidal and parallel spectra 3", f)
        self._canvas_update(self.canvas)


    def contour_poloidal_3(self):
        """contour plot of poloidal_3"""
        n_t = self._nstep
        #n_f = self._data.nfield
        x = self._data.poloidata[n_t, self._data.nfield, :, :]
        y = self._data.poloidata[n_t, self._data.nfield+1, :, :]
        f = self._data.poloidata[n_t, 2, :, :]
        # print(x.shape, y.shape, f.shape)
        # print(self._data.nfield)
        # print(self._data.poloidata.shape)

        self._pcolormesh("Poloidal_3", x, y, f)
        # self._pcolormesh("Poloidal_1", x[0,:,:], y[0,:,:], f[0,:,:])
        self._ax1.set_xlabel("R")
        self._ax1.set_ylabel("Z")
        self._ax1.set_title("Poloidal")
        self._canvas_update(self.canvas)

    def plot_radius_field_3(self):
        """Radius profile of field and rms 3"""
        n_t = self._nstep
        f = self._data.poloidata[n_t, 2, :, :]
        self._cut1d("Radius field and rms 3", f, 1)
        self._canvas_update(self.canvas)

    def plot_poloidal_field_3(self):
        """Poloidal profile of field and rms 3"""
        n_t = self._nstep
        f = self._data.poloidata[n_t, 2, :, :]
        self._cut1d("Poloidal field and rms 3", f, 2)
        self._canvas_update(self.canvas)

